
#include "convert.h"
#include "../emusic.h"

static char *Large = NULL;
static char *Small = NULL;

static void WriteButton(FILE * theme, char *name, int x, int y, int w, int h,
                        char *left, char *middle, char *right, int prelit,
                        int visible, int special)
{
   fprintf(theme, "  begin button\n");
   if (!special) {
      fprintf(theme, "    normal %s %d %d %d %d\n", name, 0, 0, w, h);
      fprintf(theme, "    clicked %s %d %d %d %d\n", name, w, 0, w, h);
      if (prelit)
         fprintf(theme, "    mouseover %s %d %d %d %d\n", name, w * 2, 0, w, h);
   }
   else {
      fprintf(theme, "    normal %s %d %d %d %d\n", name, w * 2, 0, w, h);
      fprintf(theme, "    clicked %s %d %d %d %d\n", name, w * 3, 0, w, h);
      if (prelit)
         fprintf(theme, "    mouseover %s %d %d %d %d\n", name, w * 5, 0, w, h);
   }
   fprintf(theme, "    visible %c\n    location %d %d\n", visible ? 't' : 'f',
           x, y);
   if (left)
      fprintf(theme, "    action_l %s\n", left);
   else
      fprintf(theme, "    action_l none\n");
   if (middle)
      fprintf(theme, "    action_m %s\n", middle);
   if (right)
      fprintf(theme, "    action_r %s\n", right);
   fprintf(theme, "  end\n\n");
}

static void WriteMultiButton(FILE * theme, char *name, int x, int y, int w,
                             int h, char *left, char *middle, char *right,
                             int prelit, int visible)
{
   if (left && !strcmp("repeat", left))
      fprintf(theme, "  begin repeatmode\n");
   else
      fprintf(theme, "  begin multibutton\n");
   fprintf(theme, "    visible %c\n", visible ? 't' : 'f');
   WriteButton(theme, name, x, y, w, h, left, middle, right, prelit, visible, FALSE);
   WriteButton(theme, name, x, y, w, h, left, middle, right, prelit, visible, TRUE);
   fprintf(theme, "  end\n\n");
}

static void ParseButton(FILE * theme, char *line, char *left, char *middle,
                        char *right, int form1)
{
   char s1[FILEPATH_LEN_MAX], s2[FILEPATH_LEN_MAX];
   char s[FILEPATH_LEN_MAX];
   char name[FILEPATH_LEN_MAX];
   ImageDat *tempim;
   int x, y, w, h, divider, prelit, status;

   splitstring(s1, s2, line);
   tempim = LoadImage(s1);
   if (!tempim)
      return;
   w = tempim->w;
   h = tempim->h;
   FreeImageDat(tempim);
   sprintf(name, "%s", s1);
   sprintf(s, s2);
   splitstring(s1, s2, s);
   prelit = parsebool(s1);
   if (form1) {
      sprintf(s, s2);
      splitstring(s1, s2, s);
      status = parsebool(s1);
   }
   else
      status = FALSE;
   getcoords(s2, &x, &y);
   if (form1) {
      if (prelit && status)
         divider = 6;
      else if (prelit && !status)
         divider = 3;
      else if (!prelit && status)
         divider = 4;
      else
         divider = 2;
   }
   else {
      if (prelit)
         divider = 3;
      else
         divider = 2;
   }
   w /= divider;
   if (!status)
      WriteButton(theme, name, x, y, w, h, left, middle, right, prelit, TRUE, FALSE);
   else if (status)
      WriteMultiButton(theme, name, x, y, w, h, left, middle, right, prelit, TRUE);
}

static void ParseTitle(FILE * theme, char *data, char *section)
{
   char name[FILEPATH_LEN_MAX];
   int length, x, y, w, h;
   ImageDat *tempim;

   sscanf(data, "%s %d %d %d", name, &length, &x, &y);
   if (!(tempim = LoadImage(name))) {
      fprintf(stderr, "Bad filename %s in Title: conversion.\n", name);
      return;
   }
   w = tempim->w / 32;
   h = tempim->h / 3;
   FreeImageDat(tempim);
   fprintf(theme, "  begin pixfont\n");
   fprintf(theme, "    name gqmpeg_%s\n", section);
   fprintf(theme, "    filename %s\n", name);
   fprintf(theme, "    upperlower false\n");
   fprintf(theme, "    map \" !\"#$%&'()*+,-./0123456789:;<=>?\"\n");
   fprintf(theme, "    map \"@ABCDEFGHIJKLMNOPQRSTUVWXYZ[\\]^_\"\n");
   fprintf(theme, "    map \"`abcdefghijklmnopqrstuvwxyz{|}~ \"\n");
   fprintf(theme, "  end\n\n");
   fprintf(theme, "  begin name\n");
   fprintf(theme, "    location %d %d\n", x, y);
   fprintf(theme, "    size %d %d\n", w * length, h);
   fprintf(theme, "    interactive f\n");
   fprintf(theme, "    visible t\n");
   fprintf(theme, "    fontname gqmpeg_%s\n", section);
   fprintf(theme, "    scroll t\n");
   fprintf(theme, "  end\n\n");
}

static void ParseDigits(FILE * theme, char *data, char *name, int number, int numzeros)
{
   ImageDat *tempim;
   int i;
   int x, y, w;
   char digit[FILEPATH_LEN_MAX];

   sscanf(data, "%s %d %d", digit, &x, &y);
   if (!strcasecmp("LARGE", digit) && Large)
      strcpy(digit, Large);
   else if (!strcasecmp("SMALL", digit) && Small)
      strcpy(digit, Small);
   if (!(tempim = LoadImage(digit))) {
      fprintf(stderr, "Unable to find image %s for digits\n", digit);
      return;
   }
   w = tempim->w / 11;
   FreeImageDat(tempim);
   for (i = 0; i < number; i++) {
      fprintf(theme, "  begin %s\n", name);
      fprintf(theme, "    normal %s\n", digit);
      fprintf(theme, "    location %d %d\n", x + w * i, y);
      fprintf(theme, "    parts 11\n");
      if (numzeros && (numzeros - i))
         fprintf(theme, "    display_if_zero false\n");
      fprintf(theme, "  end\n\n");
   }
}

static void Convertskindata(char *ts, char *filefrom, char *section)
{
   FILE *skindata, *theme;
   char s1[FILEPATH_LEN_MAX], s2[FILEPATH_LEN_MAX];
   char st[FILEPATH_LEN_MAX];
   char *play_save = NULL;
   int end = FALSE;
   int last = FALSE;

   sprintf(s1, "%s/%s", origskin, filefrom);
   skindata = fopen(s1, "r");
   if (!skindata) {
      fprintf(stderr, "Couldn't open '%s' file. from %s\n", filefrom, origskin);
      return;
   }
   sprintf(s1, "%s/MAIN.theme", ts);
   theme = fopen(s1, "a");
   if (!theme) {
      fprintf(stderr, "Couldn't write converted theme.\n");
      fclose(skindata);
      return;
   }
   if (!strcmp("emusic", section)) {
      fprintf(theme, "begin main\n  maskrgb 255 0 255\n");
      fprintf(theme, "  foregroundrgb 0 196 0\nend\n");
   }
   fprintf(theme, "\n\nbegin %s\n  timermode 0\n\n", section);
   if (strcmp("emusic", section))
      fprintf(theme, "  visible f\n\n");
   while (!end) {
      if (GetNextLine(st, skindata)) {
         splitstring(s1, s2, st);
         upstr(s1);
         if (last) {
            if (!strncmp("MASK:", s1, 5))
               fprintf(theme, "    mask %s\n  end\n\n", s2);
            else
               fprintf(theme, "  end\n\n");
            last = FALSE;
         }
         if (!strncmp("BACKGROUND:", s1, 11)) {
            fprintf(theme, "  begin background\n");
            fprintf(theme, "    shaped t\n    visible t\n    name eMusic\n");
            fprintf(theme, "    background %s\n", s2);
            last = TRUE;
         }
         else if (!strncmp("TITLE:", s1, 6))
            ParseTitle(theme, s2, section);
         else if (!strncmp("PLAY_BUTTON:", s1, 12))
            play_save = duplicate(s2);
         else if (!strncmp("PAUSE_BUTTON:", s1, 13))
            ParseButton(theme, s2, "pause", NULL, NULL, TRUE);
         else if (!strncmp("STOP_BUTTON:", s1, 12))
            ParseButton(theme, s2, "stop", NULL, NULL, TRUE);
         else if (!strncmp("SHUFFLE_BUTTON:", s1, 15))
            ParseButton(theme, s2, "shuffle", NULL, NULL, TRUE);
         else if (!strncmp("REPEAT_BUTTON:", s1, 14))
            ParseButton(theme, s2, "repeat", NULL, NULL, TRUE);
         else if (!strncmp("PREV_BUTTON:", s1, 12))
            ParseButton(theme, s2, "downtrack", "rew", "first", FALSE);
         else if (!strncmp("NEXT_BUTTON:", s1, 12))
            ParseButton(theme, s2, "uptrack", "ff", "last", FALSE);
         else if (!strncmp("PLAYLIST_BUTTON:", s1, 16))
            ParseButton(theme, s2, "toggle_playlist", NULL, NULL, FALSE);
         else if (!strncmp("CONFIG_BUTTON:", s1, 14))
            ParseButton(theme, s2, "configure", NULL, NULL, FALSE);
         else if (!strncmp("ICONIFY_BUTTON:", s1, 15))
            ParseButton(theme, s2, "minimize", NULL, NULL, FALSE);
         else if (!strncmp("MIXER_BUTTON:", s1, 13))
            ParseButton(theme, s2, NULL, NULL, NULL, FALSE);
         else if (!strncmp("EXIT_BUTTON:", s1, 12))
            ParseButton(theme, s2, "exit", NULL, NULL, FALSE);
         else if (!strncmp("ALT_SKIN_BUTTON:", s1, 16))
            ParseButton(theme, s2, "shade", NULL, NULL, FALSE);
         else if (!strncmp("DIGIT_LARGE_DEFAULT:", s1, 20)) {
            if (Large)
               Efree(Large);
            Large = duplicate(s2);
         }
         else if (!strncmp("DIGIT_SMALL_DEFAULT:", s1, 20)) {
            if (Small)
               Efree(Small);
            Small = duplicate(s2);
         }
         else if (!strncmp("MINUTE_NUMBER:", s1, 14))
            ParseDigits(theme, s2, "time_digit", 2, 1);
         else if (!strncmp("SECOND_NUMBER:", s1, 14))
            ParseDigits(theme, s2, "time_digit", 2, 0);
         else if (!strncmp("SONG_NUMBER:", s1, 12))
            ParseDigits(theme, s2, "track_digit", 3, 2);
         else if (!strncmp("TOTAL_NUMBER:", s1, 13))
            ParseDigits(theme, s2, "total_track_digit", 3, 2);
      }
      else
         end = TRUE;
   }
   if (play_save)
      ParseButton(theme, play_save, "play", NULL, NULL, TRUE);
   fprintf(theme, "end\n");
   fclose(skindata);
   fclose(theme);
}

int CheckForGQMpegAt(char *path)
{
   char *s = NULL;

   EDBUG(4, "CheckForGQMpegAt");
   s = FindFile("skindata", path);
   if (s) {
      Efree(s);
      EDBUG_RETURN(TRUE);
   }
   EDBUG_RETURN(FALSE);
}

void ConvertGQMpeg(char *ts)
{
   char *savepath, *temp, sys[FILEPATH_LEN_MAX];

   EDBUG(4, "ConvertGQMpeg");
   savepath = cwd();
   if (strcmp(ts, origskin)) {
      sprintf(sys, "cp -ra %s/* %s/\n", origskin, ts);
      system(sys);
   }
   chdir(ts);
   ConvertSetPath(ts);
   Convertskindata(ts, "skindata", "emusic");
   if ((temp = FindFile("skindata_alt", origskin))) {
      Convertskindata(ts, "skindata_alt", "minwin");
      Efree(temp);
   }
   ConvertFreePath();
   chdir(savepath);
   Efree(savepath);
   EDBUG_RETURN_;
}
