#include "emusic.h"
#include <netdb.h>
#include <sys/stat.h>
#include <sys/socket.h>
#include <netinet/in.h>

int streaming_type = TYPE_FILE;

void parse_shoutcast(int sock)
{
   cur_item->name = duplicate("SHOUTcast junk");
   cur_item->length = 0;
}

void parse_url(const char *url, char *user, char *pass, char *host, int *port,
               char *filename)
{
   char rurl[FILEPATH_LEN_MAX];
   char *temp;

   rurl[0] = '\0';
   if (!strncasecmp("http://", url, 7))
      strcpy(rurl, url + 7);
   else if (!strncasecmp("ftp://", url, 6))
      strcpy(rurl, url + 6);
   else
      strcpy(rurl, url);
   if ((temp = strchr(rurl, ':')) && strchr(rurl, '@')) {
      temp[0] = '\0';
      strcpy(user, rurl);
      strcpy(rurl, temp + 1);
      temp = strchr(rurl, '@');
      temp[0] = '\0';
      strcpy(pass, rurl);
      strcpy(rurl, temp + 1);
   }
   else
      pass[0] = user[0] = '\0';
   if ((temp = strchr(rurl, ':'))) {
      char *temp2 = strchr(rurl, '/');
      char portx[80];

      temp[0] = '\0';
      temp++;
      if (temp2)
         strncpy(portx, temp, temp2 - temp);
      else
         strcpy(portx, temp);
      *port = atoi(portx);
      if (temp2)
         strcat(rurl, temp2);
   }
   else if (streaming_type == TYPE_FTP)
      *port = 21;
   else
      *port = 80;
   if ((temp = strchr(rurl, '/'))) {
      strcpy(filename, temp);
      temp[0] = '\0';
   }
   else
      strcpy(filename, "/");
   strcpy(host, rurl);
}

FILE *emusic_fopen(char *path, char *mode)
{
   int do_shoutcast = FALSE;

   EDBUG(2, "emusic_fopen");
   if (streaming_type == TYPE_SHOUT) {
      streaming_type = TYPE_HTTP;
      do_shoutcast = TRUE;
   }
   if (streaming_type == TYPE_FILE) {
      EDBUG_RETURN(fopen(path, mode));
   }
   else if (streaming_type == TYPE_HTTP) {
      unsigned long addr;
      struct hostent *hp;
      struct sockaddr_in address;
      int address_len, our_socket;
      int server_port;
      char server_address[FILEPATH_LEN_MAX];
      char user[80];
      char pass[80];
      char filename[FILEPATH_LEN_MAX];
      char buffer[FILEPATH_LEN_MAX];

      if ((our_socket = socket(AF_INET, SOCK_STREAM, 0)) < 0)
         EDBUG_RETURN(NULL);
      address.sin_family = AF_INET;
      address_len = sizeof(struct sockaddr_in);

      parse_url(path, user, pass, server_address, &server_port, filename);
      if ((hp = gethostbyname(server_address)) == NULL || !*(hp->h_addr_list))
         EDBUG_RETURN(NULL);
      sprintf(buffer, "GET %s HTTP/1.0\n\n", filename);
      memcpy(&address.sin_addr.s_addr, *(hp->h_addr_list), sizeof(address.sin_addr.s_addr));
      address.sin_port = htons(server_port);
      if (connect(our_socket, (struct sockaddr *) &address, address_len) < 0)
         EDBUG_RETURN(NULL);
      write(our_socket, buffer, strlen(buffer));
      if (do_shoutcast)
         parse_shoutcast(our_socket);
      else if (!cur_item->name) 
         cur_item->name = duplicate(path);
      EDBUG_RETURN(fdopen(our_socket, mode));
   }
   EDBUG_RETURN(NULL);
}

int emusic_fclose(FILE * stream)
{
   EDBUG(2, "emusic_fclose");
   switch (streaming_type) {
   case TYPE_FILE:
   case TYPE_HTTP:
      EDBUG_RETURN(fclose(stream));
   default:
      EDBUG_RETURN(0);
   }
}

size_t emusic_fread(void *ptr, size_t size, size_t nmemb, FILE * stream)
{
   EDBUG(2, "emusic_fread");
   switch (streaming_type) {
   case TYPE_FILE:
   case TYPE_HTTP:
      EDBUG_RETURN(fread(ptr, size, nmemb, stream));
   default:
      EDBUG_RETURN(0);
   }
}

int emusic_fseek(FILE * stream, long offset, int whence)
{
   EDBUG(2, "emusic_fseek");
   if (streaming_type == TYPE_FILE)
      EDBUG_RETURN(fseek(stream, offset, whence));
   EDBUG_RETURN(-1);
}

int emusic_getc(FILE * stream)
{
   EDBUG(2, "emusic_getc");
   switch (streaming_type) {
   case TYPE_FILE:
   case TYPE_HTTP:
      EDBUG_RETURN(getc(stream));
   default:
      EDBUG_RETURN(-1);
   }
}

int emusic_feof(FILE * stream)
{
   EDBUG(2, "emusic_feof");
   switch (streaming_type) {
   case TYPE_FILE:
   case TYPE_HTTP:
      EDBUG_RETURN(feof(stream));
   default:
      EDBUG_RETURN(-1);
   }
}

long emusic_ftell(FILE * stream)
{
   EDBUG(2, "emusic_ftell");
   if (streaming_type == TYPE_FILE)
      EDBUG_RETURN(ftell(stream));
   EDBUG_RETURN(0);
}

long emusic_getsize(char *filename)
{
   EDBUG(2, "emusic_getsize");
   if (streaming_type == TYPE_FILE) {
      EDBUG_RETURN(filesize(filename));
   }
   EDBUG_RETURN(0);
}
