
#include <gtk/gtk.h>
#define USE_XIM
#include <gdk/gdkprivate.h>
#include <gdk/gdkx.h>
#include "widget.h"
#include <gmodule.h>
#include <X11/keysym.h>
#include "../plugin.h"
#include "../playlist.h"

pthread_t gtk_thread;
static int gtk_thread_ready_for_action = FALSE;
static int KillOffGtkThread = FALSE;

int MessageBox = FALSE;
int MessageBoxDraw = FALSE;
int InfoEdit = FALSE;
int InfoEditDraw = FALSE;
int AboutBox = FALSE;
int AboutBoxDraw = FALSE;
int FileBoxDraw = FALSE;
int SinglePlayDraw = FALSE;
struct playlist_item *EditChange = NULL;
char *MessageBoxLabel = NULL;

extern char *HomeDir;

static void Find_gtkrc(void)
{
   char *filename = NULL;

   EDBUG(6, "Find_gtkrc");
   if (exists("/usr/local/share/gtkrc"))
      gtk_rc_parse("/usr/local/share/gtkrc");
   if (exists("/usr/share/gtkrc"))
      gtk_rc_parse("/usr/share/gtkrc");
   filename = Emalloc(strlen(HomeDir) + strlen("/.gnome/gtkrc") + 1);
   strcpy(filename, HomeDir);
   strcat(filename, "/.gnome/gtkrc");
   if (exists(filename))
      gtk_rc_parse(filename);
   filename = Erealloc(filename, strlen(HomeDir) + strlen("/.emusic/.gtkrc") + 1);
   strcpy(filename, HomeDir);
   strcat(filename, "/.emusic/.gtkrc");
   if (exists(filename))
      gtk_rc_parse(filename);
   Efree(filename);
   EDBUG_RETURN_;
}

typedef struct {
   int *argc;
   char ***argv;
} argstruct;

static gint message_delete_event(GtkWidget * widget, GdkEvent * event, gpointer data)
{
   MessageBox = -1;
   return FALSE;
}

static void message_cancel_event(GtkWidget * widget, GtkWidget * dialog)
{
   MessageBox = -1;
   gtk_widget_destroy(dialog);
}

static void message_ok_event(GtkWidget * widget, GtkWidget * dialog)
{
   MessageBox = 0;
   gtk_widget_destroy(dialog);
}

static void message_destroy(GtkWidget * widget, gpointer data)
{
}

static void PopUpMessageBox(char *MessageBoxLabel)
{
   GtkWidget *window;
   GtkWidget *vbox;
   GtkWidget *hbox;
   GtkWidget *message;
   GtkWidget *ok_button;
   GtkWidget *cancel_button;

   window = gtk_window_new(GTK_WINDOW_TOPLEVEL);
   gtk_signal_connect(GTK_OBJECT(window), "delete_event",
                      GTK_SIGNAL_FUNC(message_delete_event), NULL);
   gtk_signal_connect(GTK_OBJECT(window), "destroy", 
                      GTK_SIGNAL_FUNC(message_destroy), NULL);
   gtk_container_set_border_width(GTK_CONTAINER(window), 10);

   vbox = gtk_vbox_new(FALSE, 0);
   gtk_container_set_border_width(GTK_CONTAINER(vbox), 0);
   gtk_container_add(GTK_CONTAINER(window), vbox);
   gtk_widget_show(vbox);

   message = gtk_label_new((gchar *) MessageBoxLabel);
   gtk_label_set_line_wrap(GTK_LABEL(message), TRUE);
   gtk_container_add(GTK_CONTAINER(vbox), message);
   gtk_widget_show(message);

   hbox = gtk_hbox_new(FALSE, 10);
   gtk_container_set_border_width(GTK_CONTAINER(hbox), 5);
   gtk_container_add(GTK_CONTAINER(vbox), hbox);
   gtk_widget_show(hbox);

   ok_button = gtk_button_new_with_label("Okay");
   gtk_signal_connect(GTK_OBJECT(ok_button), "clicked",
                      GTK_SIGNAL_FUNC(message_ok_event), window);
   gtk_container_add(GTK_CONTAINER(hbox), ok_button);
   gtk_widget_show(ok_button);

   if (MessageBox == 2) {
      cancel_button = gtk_button_new_with_label("Cancel");
      gtk_signal_connect(GTK_OBJECT(cancel_button), "clicked",
                         GTK_SIGNAL_FUNC(message_cancel_event), window);
      gtk_container_add(GTK_CONTAINER(hbox), cancel_button);
      gtk_widget_show(cancel_button);
   }
   gtk_widget_show(window);
}

static int timer = 0;

gint gtk_checker(gpointer data)
{
   if (KillOffGtkThread)
      gtk_main_quit();
   if (FileBoxDraw) {
      DrawFileBox();
      FileBoxDraw = FALSE;
   }
   if (InfoEditDraw) {
      if (InfoEdit)
         Change_InfoEditor(EditChange);
      else {
         Create_InfoEditor(EditChange);
         InfoEdit = TRUE;
      }
      InfoEditDraw = FALSE;
   }
   if (MessageBoxDraw) {
      PopUpMessageBox(MessageBoxLabel);
      MessageBoxDraw = FALSE;
   }
   if (AboutBoxDraw) {
/*      PopUpAboutBox(); */
      AboutBoxDraw = FALSE;
   }
   if (SinglePlayDraw) {
      DrawSinglePlay();
      SinglePlayDraw = FALSE;
   }
   return TRUE;
}

void *gtk_thread_loop(void *arg_thingies)
{
   argstruct *args = (argstruct *) arg_thingies;

   EDBUG(7, "gtk_thread_loop");
   pthread_setcancelstate(PTHREAD_CANCEL_ENABLE, NULL);
   pthread_setcanceltype(PTHREAD_CANCEL_DEFERRED, NULL);
   gtk_set_locale();
   gtk_init(args->argc, args->argv);
   gdk_im_close();
   Find_gtkrc();
   timer = g_timeout_add(100, gtk_checker, NULL);
   gtk_thread_ready_for_action = TRUE;
   gtk_main();
   EDBUG_RETURN(NULL);
}

void Start_gtk_thread(int *argc, char **argv[])
{
   argstruct args;

   EDBUG(9, "Start_gtk_thread");
   args.argc = argc;
   args.argv = argv;
   pthread_create(&gtk_thread, NULL, gtk_thread_loop, (void *) &args);
   while (!gtk_thread_ready_for_action)
      usleep(5);
   EDBUG_RETURN_;
}

int DrawMessageBox(int modal, char *message, int cancelbutton)
{
   fprintf(stderr, "i've been called for some reason..\n");
   MessageBoxLabel = duplicate(message);
   MessageBox = 1 + cancelbutton;
   MessageBoxDraw = TRUE;
   if (modal || cancelbutton) {
      while (MessageBox > 0)
         usleep(50);
      Efree(MessageBoxLabel);
      MessageBoxLabel = NULL;
      return MessageBox;
   }
   Efree(MessageBoxLabel);
   MessageBoxLabel = NULL;
   return -1;
}

void DrawAboutBox(void)
{
   if (!AboutBox) {
      AboutBox = TRUE;
      AboutBoxDraw = TRUE;
      return;
   }
}

void DrawInfoEditor(int modal, struct playlist_item *item_to_edit)
{
   EditChange = item_to_edit;
   InfoEditDraw = TRUE;
   if (modal)
      while (InfoEdit || InfoEditDraw)
         usleep(50);
}

void KillGtkThread(void)
{
   KillOffGtkThread = TRUE;
   pthread_join(gtk_thread, NULL);
}
