
#include "widget.h"
#include "../fileops.h"
#include "../econfig.h"

ImageDat *LoadImage(char *file)
{
   ImlibImage *im;
   char s1[FILEPATH_LEN_MAX], s2[FILEPATH_LEN_MAX];
   int crop_x, crop_y, crop_w = 0, crop_h = 0;
   ImageDat *image = Emalloc(sizeof(ImageDat));

   EDBUG(6, "LoadImage");
   splitstring(s1, s2, file);
   if (file[0] != '/') {
      char temp[FILEPATH_LEN_MAX];
      char *path = cwd();

      sprintf(temp, "%s/%s", path, s1);
      strcpy(s1, temp);
      Efree(path);
   }
   if (s2[0])
      getgeom(s2, &crop_x, &crop_y, &crop_w, &crop_h);
   if ((im = Imlib_load_image(imlib, s1)) == NULL) {
      fprintf(stderr, "Unable to load image %s.\n", s1);
      exit(-1);
   }
   if (crop_w && crop_h)
      Imlib_crop_image(imlib, im, crop_x, crop_y, crop_w, crop_h);
   Imlib_set_image_shape(imlib, im, &imask);
   Imlib_render(imlib, im, im->rgb_width, im->rgb_height);
   image->im = im;
   image->w = im->rgb_width;
   image->h = im->rgb_height;
   image->name = duplicate(s1);
   EDBUG_RETURN(image);
}

void FreeImageDat(ImageDat * dat)
{
   EDBUG(5, "FreeImageDat");
   if (!dat)
      EDBUG_RETURN_;
   Efree(dat->name);
   dat->name = NULL;
   Imlib_kill_image(imlib, dat->im);
   Efree(dat);
   dat = NULL;
   EDBUG_RETURN_;
}

ImageList *ConfigImageList(FILE * file)
{
   char st[FILEPATH_LEN_MAX];
   char s1[FILEPATH_LEN_MAX], s2[FILEPATH_LEN_MAX];
   int end = FALSE;
   int count;
   int delta = 0, middle_peak = FALSE;
   ImageList *list = Emalloc(sizeof(ImageList));

   EDBUG(5, "ConfigImageList");
   list->number = 0;
   list->images = NULL;
   while (!end) {
      if (GetNextLine(st, file)) {
         splitstring(s1, s2, st);
         upstr(s1);
         if (!strncmp("END", s1, 3))
            end = TRUE;
         else if (!strncmp("NUMBER", s1, 6)) {
            int i;

            list->number = atoi(s2);
            list->images = Emalloc(list->number * sizeof(ImageList));
            for (i = 0; i < list->number; i++);
            list->images[0] = NULL;
         }
         else if (!strncmp("MIDDLE_PEAK", s1, 11))
            middle_peak = parsebool(s2);
         else if (!strncmp("DELTA", s1, 5))
            delta = atoi(s2);
         else if (!strncmp("IMAGE", s1, 5)) {
            int x, y, w, h;
            char temp_s1[FILEPATH_LEN_MAX];
            char temp_s2[FILEPATH_LEN_MAX];
            char s3[FILEPATH_LEN_MAX];
            int mult;
            ImageDat *temp_image;

            if (!list->number) {
               fprintf(stderr, "number must preceed images in ImageList\n");
               exit(-1);
            }
            if (!delta) {
               fprintf(stderr, "unknown size (delta) in ImageList\n");
               exit(-1);
            }
            splitstring(temp_s1, temp_s2, s2);
            getgeom(temp_s2, &x, &y, &w, &h);
            temp_image = LoadImage(temp_s1);
            temp_image->im = Imlib_clone_image(imlib, temp_image->im);
            if (middle_peak) {
               for (count = 0; count < (list->number / 2); count++) {
                  mult = count;
                  if (temp_image->im->rgb_width < temp_image->im->rgb_height)
                     sprintf(s3, "%s %d %d %d %d", temp_s1, x, y + delta *
                             mult, w, h);
                  else
                     sprintf(s3, "%s %d %d %d %d", temp_s1, x + delta * mult,
                             y, w, h);
                  list->images[(list->number / 2) - count - 1] = LoadImage(s3);
                  list->images[(list->number / 2) + count] = LoadImage(s3);
               }
            }
            else {
               for (count = 0; count < list->number; count++) {
                  mult = count;
                  if (temp_image->im->rgb_width < temp_image->im->rgb_height)
                     sprintf(s3, "%s %d %d %d %d", temp_s1, x, y + delta *
                             mult, w, h);
                  else
                     sprintf(s3, "%s %d %d %d %d", temp_s1, x + delta * mult,
                             y, w, h);
                  list->images[count] = LoadImage(s3);
               }
            }
         }
         else
            fprintf(stderr, "Unknown line in ImageList config\n");
      }
      else
         end = TRUE;
   }
   EDBUG_RETURN(list);
}

void ScaleImageList(ImageList * list, double scale_w, double scale_h)
{
   int x;

   EDBUG(5, "ScaleImageList");
   for (x = 0; x < list->number; x++) {
      list->images[x]->w *= scale_w;
      list->images[x]->h *= scale_h;
      Imlib_render(imlib, list->images[x]->im, list->images[x]->w,
                   list->images[x]->h);
   }
   EDBUG_RETURN_;
}

void FreeImageList(ImageList * list)
{
   EDBUG(5, "FreeImageList");
   if (!list)
      EDBUG_RETURN_;
   Efree(list->images);
   Efree(list);
   list = NULL;
   EDBUG_RETURN_;
}
