
#include <gtk/gtk.h>
#include <gmodule.h>
#include "widget.h"
#include "../plugin.h"
#include "../playlist.h"

extern int InfoEdit;

typedef struct {
   GtkWidget *window;
   GtkWidget *title_entry;
   GtkWidget *artist_entry;
   GtkWidget *album_entry;
   GtkWidget *year_entry;
   GtkWidget *comment_entry;
   GtkWidget *ok_button;
   GtkWidget *apply_button;
   GtkWidget *omenu;
   GtkWidget *time_label;
   struct player_plugin *pplugin;
   struct playlist_item *editee;
   int changed;
} InfoEditor;

InfoEditor *infoedit = NULL;

static void Do_Apply_Info(void)
{
   gchar *text;

   if (!infoedit->changed)
      return;
   gtk_widget_set_sensitive(infoedit->ok_button, FALSE);
   gtk_widget_set_sensitive(infoedit->apply_button, FALSE);
   text = gtk_entry_get_text(GTK_ENTRY(infoedit->title_entry));
   infoedit->editee->title = duplicate(text);
   text = gtk_entry_get_text(GTK_ENTRY(infoedit->artist_entry));
   infoedit->editee->artist = duplicate(text);
   text = gtk_entry_get_text(GTK_ENTRY(infoedit->album_entry));
   infoedit->editee->album = duplicate(text);
   text = gtk_entry_get_text(GTK_ENTRY(infoedit->year_entry));
   infoedit->editee->year = duplicate(text);
   text = gtk_entry_get_text(GTK_ENTRY(infoedit->comment_entry));
   infoedit->editee->comment = duplicate(text);
   infoedit->editee->player = infoedit->pplugin;
}

static gint info_delete_event(GtkWidget * widget, GdkEvent * event, gpointer data)
{
   return FALSE;
}

static void info_destroy(GtkWidget * widget, gpointer data)
{
   InfoEdit = FALSE;
}

static void info_ok_button_event(GtkWidget * widget, GtkWidget * window)
{
   Do_Apply_Info();
   gtk_widget_destroy(window);
}

static void info_close_button_event(GtkWidget * widget, GtkWidget * window)
{
   gtk_widget_destroy(window);
}

static void info_apply_button_event(GtkWidget * widget, gpointer data)
{
   Do_Apply_Info();
}

static void text_changed_event(GtkWidget * widget, gpointer data)
{
   gtk_widget_set_sensitive(infoedit->ok_button, TRUE);
   gtk_widget_set_sensitive(infoedit->apply_button, TRUE);
   infoedit->changed = TRUE;
}

static void plugin_select_callback(GtkWidget * item, gpointer plugin)
{
   if (infoedit->pplugin != (struct player_plugin *) plugin) {
      infoedit->pplugin = (struct player_plugin *) plugin;
      text_changed_event(NULL, NULL);
   }
}

GtkWidget *make_menu_item(gchar * name, GtkSignalFunc callback, gpointer data)
{
   GtkWidget *item;

   item = gtk_menu_item_new_with_label(name);
   gtk_signal_connect(GTK_OBJECT(item), "activate", callback, data);
   gtk_widget_show(item);
   return item;
}

void Create_InfoEditor(struct playlist_item *editee)
{
   GtkWidget *vbox;
   GtkWidget *hbox;
   GtkWidget *table;
   GtkWidget *label;
   GtkWidget *menu;
   GtkWidget *item;
   GtkWidget *separator;
   GtkWidget *close_button;

   struct player_plugin *pplugin;

   if (!editee)
      return;

   if (!infoedit)
      infoedit = Emalloc(sizeof(InfoEditor));
   infoedit->window = gtk_window_new(GTK_WINDOW_TOPLEVEL);
   gtk_window_set_title(GTK_WINDOW(infoedit->window), "eMusic Info Editor");
   gtk_signal_connect(GTK_OBJECT(infoedit->window), "delete_event",
                      GTK_SIGNAL_FUNC(info_delete_event), NULL);
   gtk_signal_connect(GTK_OBJECT(infoedit->window), "destroy",
                      GTK_SIGNAL_FUNC(info_destroy), NULL);
   gtk_container_set_border_width(GTK_CONTAINER(infoedit->window), 5);

   vbox = gtk_vbox_new(FALSE, 0);
   gtk_container_add(GTK_CONTAINER(infoedit->window), vbox);
   gtk_widget_show(vbox);

   table = gtk_table_new(7, 2, FALSE);
   gtk_box_pack_start(GTK_BOX(vbox), table, FALSE, FALSE, 0);
   gtk_widget_show(table);

   /* Song title entry */
   label = gtk_label_new("Song Title:");
   gtk_misc_set_alignment(GTK_MISC(label), 1.0, 0.5);
   gtk_table_attach(GTK_TABLE(table), label, 0, 1, 0, 1, GTK_FILL, GTK_FILL,
                    10, 1);
   gtk_widget_show(label);

   infoedit->title_entry = gtk_entry_new();
   if (editee->title)
      gtk_entry_set_text(GTK_ENTRY(infoedit->title_entry), editee->title);
   gtk_signal_connect(GTK_OBJECT(infoedit->title_entry), "changed",
                      GTK_SIGNAL_FUNC(text_changed_event), NULL);
   gtk_table_attach(GTK_TABLE(table), infoedit->title_entry, 1, 2, 0, 1,
                    GTK_EXPAND | GTK_FILL, GTK_FILL, 0, 1);
   gtk_widget_show(infoedit->title_entry);

   /* Artist entry */
   label = gtk_label_new("Artist:");
   gtk_misc_set_alignment(GTK_MISC(label), 1.0, 0.5);
   gtk_table_attach(GTK_TABLE(table), label, 0, 1, 1, 2, GTK_FILL, GTK_FILL,
                    10, 1);
   gtk_widget_show(label);

   infoedit->artist_entry = gtk_entry_new();
   if (editee->artist)
      gtk_entry_set_text(GTK_ENTRY(infoedit->artist_entry), editee->artist);
   gtk_signal_connect(GTK_OBJECT(infoedit->artist_entry), "changed",
                      GTK_SIGNAL_FUNC(text_changed_event), NULL);
   gtk_table_attach(GTK_TABLE(table), infoedit->artist_entry, 1, 2, 1, 2,
                    GTK_EXPAND | GTK_FILL, GTK_FILL, 0, 1);
   gtk_widget_show(infoedit->artist_entry);

   /* Album entry */
   label = gtk_label_new("Album:");
   gtk_misc_set_alignment(GTK_MISC(label), 1.0, 0.5);
   gtk_table_attach(GTK_TABLE(table), label, 0, 1, 2, 3, GTK_FILL, GTK_FILL,
                    10, 1);
   gtk_widget_show(label);

   infoedit->album_entry = gtk_entry_new();
   if (editee->album)
      gtk_entry_set_text(GTK_ENTRY(infoedit->album_entry), editee->album);
   gtk_signal_connect(GTK_OBJECT(infoedit->album_entry), "changed",
                      GTK_SIGNAL_FUNC(text_changed_event), NULL);
   gtk_table_attach(GTK_TABLE(table), infoedit->album_entry, 1, 2, 2, 3,
                    GTK_EXPAND | GTK_FILL, GTK_FILL, 0, 1);
   gtk_widget_show(infoedit->album_entry);

   /* Year entry */
   label = gtk_label_new("Year:");
   gtk_misc_set_alignment(GTK_MISC(label), 1.0, 0.5);
   gtk_table_attach(GTK_TABLE(table), label, 0, 1, 3, 4, GTK_FILL, GTK_FILL,
                    10, 1);
   gtk_widget_show(label);

   infoedit->year_entry = gtk_entry_new();
   if (editee->year)
      gtk_entry_set_text(GTK_ENTRY(infoedit->year_entry), editee->year);
   gtk_signal_connect(GTK_OBJECT(infoedit->year_entry), "changed",
                      GTK_SIGNAL_FUNC(text_changed_event), NULL);
   gtk_table_attach(GTK_TABLE(table), infoedit->year_entry, 1, 2, 3, 4,
                    GTK_EXPAND | GTK_FILL, GTK_FILL, 0, 1);
   gtk_widget_show(infoedit->year_entry);

   /* Comment entry */
   label = gtk_label_new("Comments:");
   gtk_misc_set_alignment(GTK_MISC(label), 1.0, 0.5);
   gtk_table_attach(GTK_TABLE(table), label, 0, 1, 4, 5, GTK_FILL, GTK_FILL,
                    10, 1);
   gtk_widget_show(label);

   infoedit->comment_entry = gtk_entry_new();
   if (editee->comment)
      gtk_entry_set_text(GTK_ENTRY(infoedit->comment_entry), editee->comment);
   gtk_signal_connect(GTK_OBJECT(infoedit->year_entry), "changed",
                      GTK_SIGNAL_FUNC(text_changed_event), NULL);
   gtk_table_attach(GTK_TABLE(table), infoedit->comment_entry, 1, 2, 4, 5,
                    GTK_EXPAND | GTK_FILL, GTK_FILL, 0, 1);
   gtk_widget_show(infoedit->comment_entry);

   /* Length display */
   label = gtk_label_new("Length:");
   gtk_misc_set_alignment(GTK_MISC(label), 1.0, 0.5);
   gtk_table_attach(GTK_TABLE(table), label, 0, 1, 5, 6, GTK_FILL, GTK_FILL,
                    10, 1);
   gtk_widget_show(label);

   {
      gchar *length;
      char minutes[5];
      char seconds[5];
      int min, sec;

      min = editee->length / 60;
      sec = editee->length % 60;
      sprintf(minutes, "%02d", min);
      sprintf(seconds, "%02d", sec);
      length = g_strconcat((gchar *) minutes, ":", (gchar *) seconds, NULL);
      infoedit->time_label = gtk_label_new(length);
      gtk_table_attach(GTK_TABLE(table), infoedit->time_label, 1, 2, 5, 6,
                       GTK_EXPAND | GTK_FILL, GTK_FILL, 10, 1);
      gtk_widget_show(infoedit->time_label);
   }

   /* Plugin selection */
   label = gtk_label_new("Player:");
   gtk_misc_set_alignment(GTK_MISC(label), 1.0, 0.5);
   gtk_table_attach(GTK_TABLE(table), label, 0, 1, 6, 7, GTK_FILL, GTK_FILL,
                    10, 1);
   gtk_widget_show(label);

   infoedit->omenu = gtk_option_menu_new();
   menu = gtk_menu_new();
   {
      guint i = 0, j = 0;

      for (pplugin = players->first; pplugin; pplugin = pplugin->next) {
         item = make_menu_item((gchar *) (pplugin->id),
                               GTK_SIGNAL_FUNC(plugin_select_callback),
                               (gpointer) pplugin);
         gtk_menu_append(GTK_MENU(menu), item);
         if (editee->player == pplugin)
            j = i;
         i++;
      }
      gtk_option_menu_set_menu(GTK_OPTION_MENU(infoedit->omenu), menu);
      gtk_table_attach(GTK_TABLE(table), infoedit->omenu, 1, 2, 6, 7,
                       GTK_EXPAND | GTK_FILL, GTK_FILL, 0, 1);
      gtk_widget_show(infoedit->omenu);
      gtk_option_menu_set_history(GTK_OPTION_MENU(infoedit->omenu), j);
   }

   /* Control buttons at the bottom */
   separator = gtk_hseparator_new();
   gtk_container_add(GTK_CONTAINER(vbox), separator);
   gtk_widget_show(separator);

   hbox = gtk_hbox_new(FALSE, 10);
   gtk_container_set_border_width(GTK_CONTAINER(hbox), 5);
   gtk_container_add(GTK_CONTAINER(vbox), hbox);
   gtk_widget_show(hbox);

   infoedit->ok_button = gtk_button_new_with_label("Okay");
   gtk_signal_connect(GTK_OBJECT(infoedit->ok_button), "clicked",
                      GTK_SIGNAL_FUNC(info_ok_button_event), infoedit->window);
   gtk_container_add(GTK_CONTAINER(hbox), infoedit->ok_button);
   gtk_widget_show(infoedit->ok_button);

   infoedit->apply_button = gtk_button_new_with_label("Apply");
   gtk_signal_connect(GTK_OBJECT(infoedit->apply_button), "clicked",
                      GTK_SIGNAL_FUNC(info_apply_button_event), NULL);
   gtk_container_add(GTK_CONTAINER(hbox), infoedit->apply_button);
   gtk_widget_show(infoedit->apply_button);

   close_button = gtk_button_new_with_label("Close");
   gtk_signal_connect(GTK_OBJECT(close_button), "clicked",
                      GTK_SIGNAL_FUNC(info_close_button_event), infoedit->window);
   gtk_container_add(GTK_CONTAINER(hbox), close_button);
   gtk_widget_show(close_button);

   infoedit->editee = editee;

   gtk_widget_show(infoedit->window);

   gtk_widget_set_sensitive(infoedit->ok_button, FALSE);
   gtk_widget_set_sensitive(infoedit->apply_button, FALSE);

   infoedit->changed = FALSE;
}

void Change_InfoEditor(struct playlist_item *editee)
{
   if (!editee)
      return;
   infoedit->editee = editee;
   if (infoedit->pplugin != editee->player) {
      int i = 0;
      struct player_plugin *cur;

      for (cur = players->first; cur; cur = cur->next) {
         if (editee->player == cur)
            break;
         i++;
      }
      gtk_option_menu_set_history(GTK_OPTION_MENU(infoedit->omenu), i);
      infoedit->pplugin = editee->player;
   }
   if (editee->title)
      gtk_entry_set_text(GTK_ENTRY(infoedit->title_entry), editee->title);
   if (editee->artist)
      gtk_entry_set_text(GTK_ENTRY(infoedit->artist_entry), editee->artist);
   if (editee->album)
      gtk_entry_set_text(GTK_ENTRY(infoedit->album_entry), editee->album);
   if (editee->year)
      gtk_entry_set_text(GTK_ENTRY(infoedit->year_entry), editee->year);
   if (editee->comment)
      gtk_entry_set_text(GTK_ENTRY(infoedit->comment_entry), editee->comment);
   {
      gchar *length;
      char minutes[5];
      char seconds[5];
      int min, sec;

      min = editee->length / 60;
      sec = editee->length % 60;
      sprintf(minutes, "%02d", min);
      sprintf(seconds, "%02d", sec);
      length = g_strconcat((gchar *) minutes, ":", (gchar *) seconds, NULL);
      gtk_label_set_text(GTK_LABEL(infoedit->time_label), length);
   }
}
