
#include "widget.h"

PixFontList *pixfontlist = NULL;

PixFont *GetPixFont(char *name)
{
   PixFontList *cur = NULL;

   EDBUG(5, "GetPixFont");
   if (!pixfontlist || !pixfontlist->font)
      EDBUG_RETURN(NULL);
   cur = pixfontlist;
   while (cur) {
      if (!strcmp(name, cur->font->name))
         EDBUG_RETURN(cur->font);
      cur = cur->next;
   }
   EDBUG_RETURN(NULL);
}

static void DefaultsPixFont(PixFont * pf)
{
   int i = 0;

   EDBUG(5, "DefaultsPixFont");
   pf->font_image = NULL;
   for (; i < 256; i++) {
      pf->letter[i].row = pf->letter[i].col = 300;
      pf->letter[i].cache = NULL;
   }
   pf->letter_w = 0;
   pf->letter_h = 0;
   pf->name = NULL;
   pf->map = NULL;
   pf->nummaps = 0;
   pf->default_letter = ' ';
}

static void AddPixFontToList(PixFont * pf)
{
   PixFontList *cur;

   EDBUG(6, "AddPixFontToList");
   if (!pixfontlist) {
      pixfontlist = Emalloc(sizeof(PixFontList));
      pixfontlist->font = pf;
      pixfontlist->next = NULL;
      EDBUG_RETURN_;
   }
   cur = pixfontlist;
   while (cur->next)
      cur = cur->next;
   cur->next = Emalloc(sizeof(PixFontList));
   cur->next->font = pf;
   cur->next->next = NULL;
   EDBUG_RETURN_;
}

static void MapPixFont(PixFont * pf, int upperlower)
{
   int max_w = 0;
   int i, j;

   EDBUG(6, "MapPixFont");
   pf->letter_h = pf->font_image->h / pf->nummaps;
   for (i = 0; i < pf->nummaps; i++) {
      for (j = 0; j < strlen(pf->map[i]); j++) {
         int let = pf->map[i][j];

         if ((pf->letter[let].row == 300) && (let < 256)) {
            pf->letter[let].row = i;
            pf->letter[let].col = j;
            if (let >= 'A' && let <= 'Z' && upperlower) {
               let = let - 'A' + 'a';
               pf->letter[let].row = i;
               pf->letter[let].col = j;
            }
         }
      }
   }
   pf->letter_w = pf->font_image->w / (strlen(pf->map[0]) - 1);
   EDBUG_RETURN_;
}

void NewPixFont(FILE * file)
{
   char st[FILEPATH_LEN_MAX];
   char s1[FILEPATH_LEN_MAX], s2[FILEPATH_LEN_MAX];
   int end = FALSE, upperlower = TRUE;
   PixFont *pf = Emalloc(sizeof(PixFont));

   EDBUG(5, "NewPixfont");
   DefaultsPixFont(pf);
   while (!end) {
      if (GetNextLine(st, file)) {
         splitstring(s1, s2, st);
         upstr(s1);
         if (!strncmp("END", s1, 3))
            end = TRUE;
         else if (!strncmp("NAME", s1, 4))
            pf->name = duplicate(s2);
         else if (!strncmp("FILENAME", s1, 8))
            pf->font_image = LoadImage(s2);
         else if (!strncmp("UPPERLOWER", s1, 10))
            upperlower = parsebool(s2);
         else if (!strncmp("DEFAULT_LETTER", s1, 13))
            scanf(s2, "%c", &pf->default_letter);
         else if (!strncmp("MAP", s1, 3)) {
            char map[FILEPATH_LEN_MAX];

            pf->nummaps++;
            if (!pf->map)
               pf->map = Emalloc(sizeof(char *));

            else
               pf->map = Erealloc(pf->map, sizeof(char *) * pf->nummaps);

            if (s2[0] == '"') {
               char *temp;

               sscanf(s2, "\"%[^\n]", map);
               temp = strrchr(map, '"');
               temp[0] = '\0';
            }
            else
               strcpy(map, s2);
            pf->map[pf->nummaps - 1] = duplicate(map);
         }
      }
      else
         end = TRUE;
   }
   if (!pf->name)
      pf->name = duplicate(pf->font_image->name);
   MapPixFont(pf, upperlower);
   AddPixFontToList(pf);
   EDBUG_RETURN_;
}

void DrawPixFont(PixFont * pf, Drawable win, int x, int y, char *text)
{
   int in_string = 0, slen, dummy;
   unsigned int udummy, max_w = 0;
   Window windummy;

   EDBUG(5, "DrawPixFont");
   if (!text)
      EDBUG_RETURN_;
   while ((x + pf->letter_w) < 0) {
      x += pf->letter_w;
      in_string++;
   }
   slen = strlen(text);
   if (in_string >= slen)
      EDBUG_RETURN_;
   XGetGeometry(disp, win, &windummy, &dummy, &dummy, &max_w, &udummy, &udummy,
                &udummy);
   for (in_string; in_string < slen; in_string++) {
      ImlibImage *temp_im;
      char let = text[in_string];

      if (pf->letter[let].row == 300)
         let = pf->default_letter;
      if (!pf->letter[let].cache) {
         pf->letter[let].cache = Imlib_crop_and_clone_image(imlib,
                                                            pf->font_image->im,
                                                            pf->letter[let].col
                                                            * pf->letter_w,
                                                            pf->letter[let].row
                                                            * pf->letter_h, pf->letter_w,
                                                            pf->letter_h);
      }
      temp_im = pf->letter[let].cache;
      Imlib_paste_image(imlib, temp_im, win, x, y, pf->letter_w, pf->letter_h);
      x += pf->letter_w;
      if (x > max_w)
         break;
   }
   EDBUG_RETURN_;
}

void FreePixFont(PixFont * pf)
{
   int i;

   EDBUG(6, "FreePixFont");
   if (!pf)
      EDBUG_RETURN_;
   FreeImageDat(pf->font_image);
   Efree(pf->name);
   for (i = 0; i < pf->nummaps; i++)
      Efree(pf->map[i]);
   Efree(pf->map);
   for (i = 0; i < 256; i++)
      if (pf->letter[i].cache)
         Imlib_kill_image(imlib, pf->letter[i].cache);
   Efree(pf);
   EDBUG_RETURN_;
}

void FreePixFontList(void)
{
   PixFontList *cur, *prev;

   EDBUG(6, "FreePixFontList");
   if (!pixfontlist)
      EDBUG_RETURN_;
   cur = pixfontlist;
   while (cur) {
      prev = cur;
      cur = cur->next;
      FreePixFont(prev->font);
   }
   Efree(pixfontlist);
   pixfontlist = NULL;
   EDBUG_RETURN_;
}
