
#include <gtk/gtk.h>
#include "widget.h"
#include "../streaming.h"
#include "../control.h"
#include "../defines.h"

static int SinglePlayVisible = FALSE;
static int call_filebox = FALSE;
static int cur_type = TYPE_FILE;
static int single_ok = FALSE;
static char *single_entry = NULL;
static GtkWidget *sin_text_entry;

extern int SinglePlayDraw;
extern int keep_singles;

static void browse_event(GtkWidget * widget, gpointer data)
{
   call_filebox = TRUE;
}

static void single_ok_sel(GtkWidget * w, GtkWidget * singlewin)
{
   if (single_entry)
      Efree(single_entry);
   single_entry = duplicate(gtk_entry_get_text(GTK_ENTRY(sin_text_entry)));
   single_ok = TRUE;
   gtk_widget_destroy(GTK_WIDGET(singlewin));
}

static gint single_delete_event(GtkWidget * widget, GdkEvent * event, gpointer data)
{
   return FALSE;
}

static void single_close_sel(GtkWidget * w, GtkWidget * singlewin)
{
   gtk_widget_destroy(GTK_WIDGET(singlewin));
}

static void single_destroy(GtkWidget * w, gpointer data)
{
   SinglePlayVisible = FALSE;
}

static void file_sel(GtkWidget * w, GtkWidget * button)
{
   cur_type = TYPE_FILE;
   gtk_widget_set_sensitive(button, TRUE);
}

static void http_sel(GtkWidget * w, GtkWidget * button)
{
   cur_type = TYPE_HTTP;
   gtk_widget_set_sensitive(button, FALSE);
}

static void esd_sel(GtkWidget * w, GtkWidget * button)
{
   cur_type = TYPE_ESD;
   gtk_widget_set_sensitive(button, FALSE);
}

static void shout_sel(GtkWidget * w, GtkWidget * button)
{
   cur_type = TYPE_SHOUT;
   gtk_widget_set_sensitive(button, FALSE);
}

void DrawSinglePlay(void)
{
   GtkWidget *singleplaywindow;
   GtkWidget *button;
   GtkWidget *vbox;
   GtkWidget *hbox;
   GtkWidget *radio_button;
   GtkWidget *labels;
   GtkWidget *separator;
   GtkWidget *ok_button;
   GtkWidget *cancel_button;

   singleplaywindow = gtk_window_new(GTK_WINDOW_TOPLEVEL);
   gtk_window_set_title(GTK_WINDOW(singleplaywindow), "Single Play Selector");
   gtk_signal_connect(GTK_OBJECT(singleplaywindow), "delete_event",
                      GTK_SIGNAL_FUNC(single_delete_event), NULL);
   gtk_signal_connect(GTK_OBJECT(singleplaywindow), "destroy",
                      GTK_SIGNAL_FUNC(single_destroy), NULL);
   gtk_container_set_border_width(GTK_CONTAINER(singleplaywindow), 5);

   vbox = gtk_vbox_new(FALSE, 0);
   gtk_container_set_border_width(GTK_CONTAINER(vbox), 0);
   gtk_container_add(GTK_CONTAINER(singleplaywindow), vbox);
   gtk_widget_show(vbox);

   hbox = gtk_hbox_new(FALSE, 10);
   gtk_container_set_border_width(GTK_CONTAINER(hbox), 5);
   gtk_container_add(GTK_CONTAINER(vbox), hbox);
   gtk_widget_show(hbox);

   sin_text_entry = gtk_entry_new();
   gtk_container_add(GTK_CONTAINER(hbox), sin_text_entry);
   gtk_widget_show(sin_text_entry);

   button = gtk_button_new_with_label("Browse");
   gtk_signal_connect(GTK_OBJECT(button), "clicked",
                      GTK_SIGNAL_FUNC(browse_event), NULL);
   gtk_container_add(GTK_CONTAINER(hbox), button);
   gtk_widget_show(button);
   if (cur_type != TYPE_FILE)
      gtk_widget_set_sensitive(button, FALSE);

   hbox = gtk_hbox_new(FALSE, 10);
   gtk_container_set_border_width(GTK_CONTAINER(hbox), 5);
   gtk_container_add(GTK_CONTAINER(vbox), hbox);
   gtk_widget_show(hbox);

   radio_button = gtk_radio_button_new_with_label(NULL, "Local File");
   gtk_signal_connect(GTK_OBJECT(radio_button), "clicked",
                      GTK_SIGNAL_FUNC(file_sel), button);
   gtk_box_pack_start(GTK_BOX(hbox), radio_button, TRUE, TRUE, 0);
   if (cur_type == TYPE_FILE)
      gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON(radio_button), TRUE);
   gtk_widget_show(radio_button);

   radio_button = gtk_radio_button_new_with_label(gtk_radio_button_group(
                                                                           GTK_RADIO_BUTTON(radio_button)),
                                                  "http/ftp");
   gtk_signal_connect(GTK_OBJECT(radio_button), "clicked",
                      GTK_SIGNAL_FUNC(http_sel), button);
   gtk_box_pack_start(GTK_BOX(hbox), radio_button, TRUE, TRUE, 0);
   if ((cur_type == TYPE_HTTP) || (cur_type == TYPE_FTP))
      gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON(radio_button), TRUE);
   gtk_widget_show(radio_button);

   radio_button = gtk_radio_button_new_with_label(gtk_radio_button_group(
                                                                           GTK_RADIO_BUTTON(radio_button)),
                                                  "Remote ESD");
   gtk_signal_connect(GTK_OBJECT(radio_button), "clicked",
                      GTK_SIGNAL_FUNC(esd_sel), button);
   gtk_box_pack_start(GTK_BOX(hbox), radio_button, TRUE, TRUE, 0);
   if (cur_type == TYPE_ESD)
      gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON(radio_button), TRUE);
   gtk_widget_show(radio_button);
   gtk_widget_set_sensitive(radio_button, FALSE);

   radio_button = gtk_radio_button_new_with_label(gtk_radio_button_group(
                                                                           GTK_RADIO_BUTTON(radio_button)),
                                                  "SHOUTcast");
   gtk_signal_connect(GTK_OBJECT(radio_button), "clicked",
                      GTK_SIGNAL_FUNC(shout_sel), button);
   gtk_box_pack_start(GTK_BOX(hbox), radio_button, TRUE, TRUE, 0);
   if (cur_type == TYPE_SHOUT)
      gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON(radio_button), TRUE);
   gtk_widget_show(radio_button);

   separator = gtk_hseparator_new();
   gtk_container_add(GTK_CONTAINER(vbox), separator);
   gtk_widget_show(separator);

   hbox = gtk_hbox_new(FALSE, 10);
   gtk_container_border_width(GTK_CONTAINER(hbox), 5);
   gtk_container_add(GTK_CONTAINER(vbox), hbox);
   gtk_widget_show(hbox);

   cancel_button = gtk_button_new_with_label("Cancel");
   gtk_signal_connect(GTK_OBJECT(cancel_button), "clicked",
                      GTK_SIGNAL_FUNC(single_close_sel), singleplaywindow);
   gtk_box_pack_end(GTK_BOX(hbox), cancel_button, FALSE, TRUE, 0);
   GTK_WIDGET_SET_FLAGS(cancel_button, GTK_CAN_DEFAULT);
   gtk_widget_show(cancel_button);

   ok_button = gtk_button_new_with_label("  Play  ");
   gtk_signal_connect(GTK_OBJECT(ok_button), "clicked",
                      GTK_SIGNAL_FUNC(single_ok_sel), singleplaywindow);
   gtk_box_pack_end(GTK_BOX(hbox), ok_button, FALSE, TRUE, 0);
   GTK_WIDGET_SET_FLAGS(ok_button, GTK_CAN_DEFAULT);
   gtk_widget_grab_default(ok_button);
   gtk_widget_show(ok_button);

   gtk_widget_show(singleplaywindow);
   SinglePlayVisible = TRUE;
}

void SingleCallback(char *return_path)
{
   if (return_path)
      gtk_entry_set_text(GTK_ENTRY(sin_text_entry), return_path);
}

void PopUpSinglePlay(void)
{
   EDBUG(4, "PopUpSinglePlay");
   single_ok = FALSE;
   SinglePlayDraw = TRUE;
   while (SinglePlayDraw || SinglePlayVisible) {
      if (call_filebox) {
         SetFileBoxCallback(SingleCallback);
         FileWin->cdmode = FALSE;
         FileWin->title = duplicate("Select a file to play.");
         PopUpFileBox();
         call_filebox = FALSE;
      }
      usleep(50);
   }
   if (single_ok) {
      struct playlist_item *new_item;

      switch (cur_type) {
      case TYPE_FILE:
         new_item = playlist_item_fromfile(single_entry);
         break;
      case TYPE_HTTP:
      case TYPE_SHOUT:
         new_item = playlist_item_fromhttp(single_entry, cur_type);
         break;
      default:
         break;
      }
      if ((!keep_singles) || (cur_type != TYPE_FILE))
         new_item->delete_after = TRUE;
      add_one(1, new_item);
      if (playing) {
         stop();
         flush_audio();
         playing = FALSE;
      }
      setcurposition(playlist, 1);
      control(PLAY);
      single_ok = FALSE;
   }
   EDBUG_RETURN_;
}
