#include <stdio.h>
#include <gmodule.h>

struct playlist_item {
   char *name;
   char *title;
   char *artist;
   char *album;
   char *year;
   char *comment;
   char *path;
   int length;
   int pos;
   int stream_type;
   int delete_after;
   struct player_plugin *player;
   struct playlist_item *next;
   struct playlist_item *prev;
};

typedef struct {
   struct playlist_item *first;
   struct playlist_item *last;
   int length;
} PlayList;

struct player_plugin {
   void (*init) (struct player_plugin * plug);
   void (*config) (char *left, char *right);
   void (*seek_to_frame) (int second);
   int (*play) (char *song);
   void (*pause_it) (void);
   void (*stop) (int playing);
   void (*get_info) (struct playlist_item * song);
   char *id;
   char *version;
   char *copyright;
   GModule *handle;
   int loaded;
   int streamable;
   char *soname;
   struct player_plugin *next;
};

#ifdef __cplusplus
extern "C" {
#endif 
   extern int setup_audio(int frequency, int stereo, int sign, int big, int sixteen);
   extern void set_bitrate(int bit);
   extern void close_audio(void);
   extern void flush_audio(void);
   extern int end_of_song;
   extern int parsebool(char *data);
   extern int audio_play(char *buffer, int size);
   extern int exists(char *s);

   extern void emusic_register_ext(char *ext, struct player_plugin *plug);

   extern FILE *emusic_fopen(char *path, char *mode);
   extern int emusic_fclose(FILE * stream);
   extern size_t emusic_fread(void *ptr, size_t size, size_t nmemb, FILE * stream);
   extern int emusic_fseek(FILE * stream, long offset, int whence);
   extern int emusic_getc(FILE * stream);
   extern int emusic_feof(FILE * stream);
   extern long emusic_ftell(FILE * stream);
   extern long emusic_getsize(char *filename);

   extern void *Emalloc(int size);
   extern void *Erealloc(void *ptr, int size);
   extern void *Ecalloc(int num, int size);
   extern void Efree(void *ptr);
   char *duplicate(char *s);

   /* stuff from ../emusic/fileops.h -- if ya need more just add em here */
   char *homedir(int uid);
   char *username(int uid);
#ifdef __cplusplus
}

#endif
