
#include "mpegsound.h"
#include "splay.h"
#include <string.h>
#include <sys/stat.h>
#include <math.h>
#include <unistd.h>
#include "../common.h"

static Soundinputstream *loader = NULL; 
static Rawtoemusic *player = NULL; 
static Mpegtoraw *server = NULL;

static char *song_path;

static int pcmperframe, frequency, maxframes;
static int wait_for_me;
static int do_downmix = 0;
static int do_mono = 0;

extern int end_of_song;
extern int end_song;
extern pthread_t player_thread;
extern pthread_mutex_t player_lock;

void sseek_to_frame(int second)
{
   int frame;

   if (!server)
      return;
   if (!frequency)
      frequency = server->getfrequency();
   if (!pcmperframe)
      pcmperframe = server->getpcmperframe();
   frame = second * frequency / pcmperframe;
   server->clearbuffer();
   server->setframe(frame);
}

void *mp3_thread(void *foo)
{
   player = new Rawtoemusic;
   player->initialize(NULL);
   server = new Mpegtoraw(loader, player);
   server->initialize(song_path);
   server->setforcetomono(do_mono);
   server->setdownfrequency(do_downmix);
   server->run(-1);
   frequency = server->getfrequency();
   pcmperframe = server->getpcmperframe();
   set_bitrate(server->getbitrate());
   pthread_setcancelstate(PTHREAD_CANCEL_ENABLE, NULL);
   pthread_setcanceltype(PTHREAD_CANCEL_DEFERRED, NULL);
   wait_for_me = 0;
   while (1) {
      pthread_mutex_lock(&player_lock);
      if (!server->run(5) && (server->geterrorcode() == SOUND_ERROR_FINISH)) {
         end_song = 1;
         pthread_mutex_unlock(&player_lock);
         while (1) {
            pthread_testcancel();
            usleep(500);
         }
      }
      pthread_mutex_unlock(&player_lock);
      usleep(5);
      pthread_testcancel();
   }
}

int splay(char *path)
{
   int err;

   end_of_song = 0;
   loader = Soundinputstream::hopen(path, &err);
   if (!loader)
      return 0;
   song_path = path;
   wait_for_me = 1;
   frequency = 0;
   pcmperframe = 0;
   pthread_create(&player_thread, NULL, mp3_thread, NULL);
   while (wait_for_me)
      usleep(5);
   return 1;
}

void spause_it(void)
{
}

void sstop(int playing)
{
   if (!server)
      return;
   pthread_cancel(player_thread);
   pthread_join(player_thread, NULL);
   delete loader;
   delete player;
   delete server;

   server = NULL;
   loader = NULL;
   player = NULL;
   frequency = 0;
   pcmperframe = 0;
}

void sget_info(struct playlist_item *song)
{
   int err;
   Soundinputstream *tloader = NULL;
   Rawtoemusic *tplayer = NULL;
   Mpegtoraw *tserver = NULL;
   char *temp;

   tloader = Soundinputstream::hopen(song->path, &err);
   if (!tloader)
      return;
   tplayer = new Rawtoemusic;
   tplayer->initialize(NULL);
   tserver = new Mpegtoraw(tloader, tplayer);
   tserver->initialize(song_path);
   temp = (char *) tserver->getname();
   if (temp && *temp) {
      if (song->title)
         Efree(song->title);
      song->title = duplicate(temp);
   }
   temp = (char *) tserver->getartist();
   if (temp && *temp) {
      if (song->artist)
         Efree(song->artist);
      song->artist = duplicate(temp);
   }
   temp = (char *) tserver->getalbum();
   if (temp && *temp) {
      if (song->album)
         Efree(song->album);
      song->album = duplicate(temp);
   }
   temp = (char *) tserver->getyear();
   if (temp && *temp) {
      if (song->year)
         Efree(song->year);
      song->year = duplicate(temp);
   }
   temp = (char *) tserver->getcomment();
   if (temp && *temp) {
      if (song->comment)
         Efree(song->comment);
      song->comment = duplicate(temp);
   }
   song->length = (int) ((float) tserver->gettotalframe() *
                         (float) tserver->getpcmperframe() /
                         (float) tserver->getfrequency());
   delete tloader;
   delete tplayer;
   delete tserver;
}

void sinit(struct player_plugin *plug)
{
   static int registered = FALSE;
   
   if (!registered) {
      registered = TRUE;
      emusic_register_ext("mp3", plug);
      emusic_register_ext("mp2", plug);
      emusic_register_ext("mpg", plug);
   }
}

void sconfig(char *left, char *right)
{
   if (!strncmp("BEGIN", left, 5) || !strncmp("END", left, 3))
      return;
   else if (!strncmp("DOWNMIX", left, 7))
      do_downmix = parsebool(right);
   else if (!strncmp("FORCETOMONO", left, 4))
      do_mono = parsebool(right);
}
