/* Extended Module Player 
 * Copyright (C) 1996-1999 Claudio Matsuoka and Hipolito Carraro Jr.
 * $Id: driver.c,v 1.2 1998/10/26 20:46:11 claudio Exp $.
 * 
 * This file is part of the Extended Module Player and is distributed
 * under the terms of the GNU General Public License. See docs/COPYING
 * for more information.
 */

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include <stdlib.h>
#include <string.h>
#include "driver.h"
#include "mixer.h"
#include "period.h"
#include "adpcm.h"

int __mode_fm = 0;

struct voice_info {
   int chn;                        /* channel link */
   int root;                        /* */
   unsigned int age;                /* */
   int note;                        /* */
   int pan;                        /* */
   int vol;                        /* */
   int period;                        /* current period */
   int pbase;                        /* base period */
   int itpt;                        /* interpolation */
   int pos;                        /* position in sample */
   int fidx;                        /* function index */
   int fxor;                        /* function xor control */
   int ins;                        /* instrument number */
   int smp;                        /* sample number */
   int end;                        /* loop end */
   int freq;
   int shl8;
   int act;                        /* nna info & status of voice */
   void *sptr;                        /* sample pointer */
};

struct xmp_control *xmp_ctl;

static int *ch2vo_num = NULL;
static int *ch2vo_array = NULL;

static struct patch_info **patch_array = NULL;
static struct voice_info *voice_array = NULL;
static struct xmp_drv_info *drv_array = NULL;
static struct xmp_drv_info *driver = NULL;

static int *cmute_array = NULL;

static int numusr;                /* number of user channels */
static int numvoc;                /* number of soundcard voices */
static int numchn;                /* number of channels need to play one module */
static int numtrk;                /* number of tracks in module */
static int nummte;                /* number of channels in cmute_array */
static int maxvoc;                /* */
static int cntvoc;                /* */
static int extern_drv;                /* set output to driver other than mixer */

static int drv_select(char *);
static inline void drv_resetvoice(int);
static int patch_num = 0;

extern struct xmp_drv_info drv_emusic;

#ifdef DRIVER_OSS_SEQ
extern struct xmp_drv_info drv_oss_seq;

#endif

void (*xmp_event_callback) (unsigned long);

#include "mixer.c"


void xmp_init_drivers()
{
   xmp_drv_register(&drv_emusic);
#ifdef DRIVER_OSS_SEQ
   xmp_drv_register(&drv_oss_seq);
#endif
}


void xmp_drv_register(struct xmp_drv_info *drv)
{
   if (!drv_array)
      drv_array = drv;
   else {
      struct xmp_drv_info *tmp;

      for (tmp = drv_array; tmp->next; tmp = tmp->next);
      tmp->next = drv;
   }
   drv->next = NULL;
}

static int drv_select(char *id)
{
   int tmp;
   struct xmp_drv_info *drv;

   if (!drv_array)
      return XMP_ERR_DNREG;

   xmp_ctl->memavl = 0;

   if (!id)
      id = "emusic";

   if (id) {
      tmp = XMP_ERR_NODRV;
      for (drv = drv_array; drv; drv = drv->next)
         if (!strcmp(drv->id, id))
            if ((tmp = drv->init(xmp_ctl)) == XMP_OK)
               break;
   }
   else {
      tmp = XMP_ERR_DSPEC;
      for (drv = drv_array->next; drv; drv = drv->next) {
         if (xmp_ctl->verbose > 2)
            report("Probing %s... ", drv->description);
         if (drv->init(xmp_ctl) == XMP_OK) {
            if (xmp_ctl->verbose > 2)
               report("found\n");
            tmp = XMP_OK;
            break;
         }
         if (xmp_ctl->verbose > 2)
            report("not found\n");
      }
   }

   if (tmp != XMP_OK)
      return tmp;

   xmp_ctl->drv_id = drv->id;
   xmp_ctl->description = drv->description;
   xmp_ctl->help = drv->help;
   driver = drv;

   return XMP_OK;
}

int xmp_drv_mutelloc(int num)
{
   if ((cmute_array = Ecalloc(num, sizeof(int))) == NULL)
        return XMP_ERR_ALCMA;

   nummte = num;
   return XMP_OK;
}

int xmp_drv_open(struct xmp_control *ctl)
{
   int status;

   if ((xmp_ctl = ctl) == NULL)        /* set internal control ptr */
      return XMP_ERR_NCTRL;

   extern_drv = 1;
   smix_tbuffer = NULL;
   if ((status = drv_select(ctl->drv_id)) != XMP_OK)
      return status;

   if ((patch_array = Ecalloc(XMP_DEF_MAXPAT,
                              sizeof(struct patch_info *))) == NULL) {
      driver->shutdown();
      xmp_ctl = NULL;
      return XMP_ERR_ALPTA;
   }

   patch_num = 0;

   return XMP_OK;
}

void xmp_drv_close()
{
   xmp_drv_off();
   Efree(cmute_array);
   Efree(patch_array);
   driver->shutdown();
   cmute_array = NULL;
   xmp_ctl = NULL;
   nummte = 0;
}

int xmp_drv_on(int num)
{                                /* num = number of tracks in module */
   if (!xmp_ctl)
      return XMP_ERR_DOPEN;

   xmp_ctl->numtrk = numtrk = num;
   numusr = xmp_ctl->numusr;
   num = driver->numvoices(driver->numvoices(135711));
   driver->reset();

   maxvoc = (xmp_ctl->flags & XMP_CTL_VIRTUAL) ? xmp_ctl->maxvoc : 1;
   numtrk += numusr;
   numchn = numtrk;
   if (maxvoc != 1)
      numchn += num;
   else if (num > numchn)
      num = numchn;

   num = numvoc = driver->numvoices(num);

   if ((voice_array = Ecalloc(num, sizeof(struct voice_info))) == NULL)
        return XMP_ERR_ALLOC;
   if ((ch2vo_array = Ecalloc(numchn, sizeof(int))) == NULL) {
      Efree(voice_array);
      return XMP_ERR_ALLOC;
   }
   if ((ch2vo_num = Ecalloc(numchn, sizeof(int))) == NULL) {
      Efree(voice_array);
      Efree(ch2vo_array);
      return XMP_ERR_ALLOC;
   }

   for (; num--; voice_array[num].chn = -1, voice_array[num].root = -1);
   for (num = numchn; num--; ch2vo_array[num] = -1);        /* set chn to free */

   xmp_ctl->numvoc = cntvoc = 0;
   xmp_ctl->numchn = numchn;
   return XMP_OK;
}


void xmp_drv_off()
{
   if (numchn <= 0)
      return;

   xmp_drv_writepatch(NULL);
   xmp_ctl->numchn = xmp_ctl->numvoc = 0;
   numvoc = numtrk = numchn = 0;
   Efree(ch2vo_num);
   Efree(ch2vo_array);
   Efree(voice_array);
}

void xmp_drv_reset()
{
   int num;

   if (numchn <= 0)
      return;

   driver->numvoices(driver->numvoices(43210));
   driver->reset();
   driver->numvoices(num = numvoc);

   memset(ch2vo_num, 0, sizeof(int) * numchn);
   memset(voice_array, 0, sizeof(struct voice_info) * numvoc);

   for (; num--; voice_array[num].chn = -1, voice_array[num].root = -1);
   for (num = numchn; num--; ch2vo_array[num] = -1);
   xmp_ctl->numvoc = 0;
   cntvoc = 0;
}


void xmp_drv_clearmem()
{
   driver->clearmem();
}


void xmp_drv_resetchannel(int chn)
{
   int voc;

   chn += numusr;
   if ((uint32) chn >= numchn || (uint32) (voc = ch2vo_array[chn]) >= numvoc)
      return;

   xmp_ctl->numvoc -= 1;
   ch2vo_array[chn] = -1;
   --ch2vo_num[voice_array[voc].root];
   memset(&voice_array[voc], 0, sizeof(struct voice_info));

   voice_array[voc].root = -1;
   voice_array[voc].chn = -1;

   driver->setvol(voc, 0);
}

static void drv_resetvoice(int voc)
{
   if ((uint32) voc >= numvoc)
      return;

   xmp_ctl->numvoc -= 1;
   ch2vo_array[voice_array[voc].chn] = -1;
   --ch2vo_num[voice_array[voc].root];
   memset(&voice_array[voc], 0, sizeof(struct voice_info));

   voice_array[voc].root = -1;
   voice_array[voc].chn = -1;

   driver->setvol(voc, 0);
}


static int drv_allocvoice(int chn)
{
   int voc, vfree;
   uint32 age;

   if (ch2vo_num[chn] < maxvoc) {
      for (voc = numvoc; voc-- && voice_array[voc].chn >= 0;);
      if (voc < 0)
         return voc;

      voice_array[voc].age = cntvoc++;
      xmp_ctl->numvoc++;
      ch2vo_num[chn]++;
      return voc;
   }

   for (voc = numvoc, vfree = -1, age = (int) -1; voc--;)
      if (voice_array[voc].root == chn && voice_array[voc].age < age) {
         age = voice_array[voc].age;
         vfree = voc;
      }

   ch2vo_array[voice_array[vfree].chn] = -1;
   voice_array[vfree].age = cntvoc++;
   return vfree;
}


void xmp_drv_mute(int chn, int status)
{
   chn += numusr;
   if ((uint32) chn >= nummte)
      return;

   if (status < 0)
      cmute_array[chn] = !cmute_array[chn];
   else
      cmute_array[chn] = status;
}


void xmp_drv_setsmp(int chn, int smp)
{
   int voc, vocpos, vocitp, vocnte;
   struct voice_info *vi;

   chn += numusr;
   if ((uint32) chn >= numchn || (uint32) (voc = ch2vo_array[chn]) >= numvoc)
      return;

   vi = &voice_array[voc];
   if ((uint32) smp >= XMP_DEF_MAXPAT || !patch_array[smp] || vi->smp == smp)
      return;

   vocpos = vi->pos;
   vocitp = vi->itpt;
   vocnte = vi->note;

   smix_setpatch(voc, smp);
   vi->note = vocnte;
   if (vi->fidx & FLAG_16_BITS)
      vocpos <<= 1;
   smix_voicepos(voc, vocpos, vocitp);
   if (extern_drv) {
      driver->setpatch(voc, smp);
      driver->setnote(voc, vi->note);
      driver->voicepos(voc, vocpos);
   }
}


int xmp_drv_setpatch(int chn, int ins, int smp, int nte, int nna, int dct, int dca)
{
   int voc, vfree;

   chn += numusr;
   if ((uint32) chn >= numchn)
      return -numusr - 1;
   if (ins < 0 || (uint32) smp >= XMP_DEF_MAXPAT || !patch_array[smp])
      smp = -1;

   if (dct) {
      for (voc = numvoc; voc--;)
         if (voice_array[voc].root == chn && voice_array[voc].ins == ins)
            if ((dct == XXM_DCT_INST) ||
                (dct == XXM_DCT_SMP && voice_array[voc].smp == smp) ||
                (dct == XXM_DCT_NOTE && voice_array[voc].note == nte)) {
               if (dca) {
                  if (voc != ch2vo_array[chn] || voice_array[voc].act)
                     voice_array[voc].act = dca;
               }
               else
                  drv_resetvoice(voc);
            }
   }

   voc = ch2vo_array[chn];
   if (voc < 0) {
      if ((voc = drv_allocvoice(chn)) < 0)
         return -numusr - 1;
      voice_array[ch2vo_array[chn] = voc].chn = chn;
      voice_array[voc].root = chn;
   }
   else if (voice_array[voc].act && maxvoc != 1) {
      if ((vfree = drv_allocvoice(chn)) >= 0) {
         voice_array[vfree].root = chn;
         voice_array[ch2vo_array[chn] = vfree].chn = chn;
         for (chn = numtrk; ch2vo_array[chn++] >= 0;);
         voice_array[voc].chn = --chn;
         ch2vo_array[chn] = voc;
         voc = vfree;
      }
   }

   if (smp < 0) {
      drv_resetvoice(voc);
      return -numusr - 1;
   }
   smix_setpatch(voc, smp);
   smix_setnote(voc, nte);
   voice_array[voc].ins = ins;
   voice_array[voc].act = nna;
   if (extern_drv) {
      driver->setpatch(voc, smp);
      driver->setnote(voc, nte);
   }

   return chn - numusr;
}


void xmp_drv_setnna(int chn, int nna)
{
   int voc;

   chn += numusr;
   if ((uint32) chn >= numchn || (uint32) (voc = ch2vo_array[chn]) >= numvoc)
      return;

   voice_array[voc].act = nna;
}


void xmp_drv_pastnote(int chn, int ins, int act)
{
   int voc;

   chn += numusr;
   if ((uint32) chn >= numchn || (uint32) (voc = ch2vo_array[chn]) >= numvoc)
      return;

   for (voc = numvoc; voc--;)
      if (voice_array[voc].root == chn && voice_array[voc].ins == ins) {
         if (act)
            voice_array[voc].act = act;
         else
            drv_resetvoice(voc);
      }
}


void xmp_drv_retrig(int chn)
{
   int voc;

   chn += numusr;
   if ((uint32) chn >= numchn || (uint32) (voc = ch2vo_array[chn]) >= numvoc)
      return;

   smix_setnote(voc, voice_array[voc].note);
   if (extern_drv)
      driver->setnote(voc, voice_array[voc].note);
}


void xmp_drv_setvol(int chn, int vol)
{
   int voc;

   chn += numusr;
   if ((uint32) chn >= numchn || (uint32) (voc = ch2vo_array[chn]) >= numvoc)
      return;

   if (nummte > voice_array[voc].root)
      driver->setvol(voc, cmute_array[voice_array[voc].root] ? 0 : vol);
   else
      driver->setvol(voc, vol);

   if (!vol && chn >= numtrk)
      drv_resetvoice(voc);
}


void xmp_drv_setbend(int chn, int bend)
{
   int voc;

   chn += numusr;
   if ((uint32) chn >= numchn || (uint32) (voc = ch2vo_array[chn]) >= numvoc)
      return;

   smix_setbend(voc, bend);
   if (extern_drv)
      driver->setbend(voc, bend);
}


void xmp_drv_setpan(int chn, int pan)
{
   int voc;

   chn += numusr;
   if ((uint32) chn >= numchn || (uint32) (voc = ch2vo_array[chn]) >= numvoc)
      return;

   driver->setpan(voc, pan);
}


void xmp_drv_seteffect(int chn, int type, int val)
{
   int voc;

   chn += numusr;
   if ((uint32) chn >= numchn || (uint32) (voc = ch2vo_array[chn]) >= numvoc)
      return;

   driver->seteffect(voc, type, val);
}


void xmp_drv_voicepos(int chn, int index)
{
   int voc;

   chn += numusr;
   if ((uint32) chn >= numchn || (uint32) (voc = ch2vo_array[chn]) >= numvoc)
      return;

   smix_voicepos(voc, index, 0);
   if (extern_drv)
      driver->voicepos(voc, index);
}


int xmp_drv_cstat(int chn)
{
   int voc;

   chn += numusr;
   if ((uint32) chn >= numchn || (uint32) (voc = ch2vo_array[chn]) >= numvoc)
      return XMP_CHN_DUMB;

   return chn < numtrk ? XMP_CHN_ACTIVE : voice_array[voc].act;
}


void xmp_drv_echoback(int msg)
{
   driver->echoback(msg);
}


int xmp_drv_getmsg()
{
   return driver->getmsg();
}


void xmp_drv_bufdump()
{
   driver->bufdump(softmixer());
}


void xmp_drv_starttimer()
{
   xmp_drv_sync(0);
   driver->starttimer();
}


void xmp_drv_stoptimer()
{
   int voc;

   for (voc = numvoc; voc--;)
      driver->setvol(voc, 0);
   driver->stoptimer();

   xmp_drv_bufdump();
}


double xmp_drv_sync(double step)
{
   static double next_time = 0;

   if (step == 0)
      next_time = step;
   driver->sync(next_time += step);

   return next_time;
}


void xmp_drv_bufwipe()
{
   driver->bufwipe();
}


int xmp_drv_writepatch(struct patch_info *patch)
{
   int num;

   if (!xmp_ctl)
      return XMP_ERR_DOPEN;

   if (!patch) {
      driver->writepatch(patch);

      for (num = XMP_DEF_MAXPAT; num--;) {
         if (patch_array[num])
            Efree(patch_array[num]);
         patch_array[num] = NULL;
      }
      return XMP_OK;
   }
   if (patch->instr_no >= XMP_DEF_MAXPAT)
      return XMP_ERR_PATCH;

   patch_array[patch->instr_no] = patch;
   patch_num = patch->instr_no + 1;

   return XMP_OK;
}


int xmp_drv_cvt8bit(void)
{
   int i, j;
   struct patch_info *pi;
   int8 *p8;
   int16 *p16;

   for (i = 0; i < xxh->smp; i++) {
      if (xxs[i].flg & WAVE_16_BITS) {
         xxs[i].len >>= 1;
         pi = patch_array[i];
         p8 = (int8 *) pi->data;
         p16 = (int16 *) pi->data;
         for (j = xxs[i].len; j--; *p8++ = *p16++ >> 8);
         pi = Erealloc(pi, sizeof(struct patch_info) + xxs[i].len);

         patch_array[i] = pi;
      }
   }
   return 0;
}


int xmp_drv_flushpatch(int ratio)
{
   struct patch_info *patch;
   int c, i, num;

   if (!ratio) {
      for (i = 0; i < patch_num; i++)
         driver->writepatch(patch);
      return XMP_OK;
   }
/* So para sequencer, ver nos flags do device */
   if (xmp_ctl->verbose)
      report("Uploading smps : %d ", patch_num);
   for (i = 0; i < patch_num; i++) {
      patch = patch_array[i];
      if (!patch)
         continue;
      c = xmp_drv_crunch(&patch, ratio);
      patch_array[i] = patch;
      if ((num = driver->writepatch(patch)) != XMP_OK)
         return num;
      if (extern_drv)
         patch = Erealloc(patch, sizeof(struct patch_info));

      if (xmp_ctl->verbose)
         report(c != 0x10000 ? "c" : ".");
   }
   if (xmp_ctl->verbose)
      report("\n");

   return XMP_OK;
}


void xmp_drv_loadpatch(FILE * f, int id, int basefreq, int flags,
                       struct xxm_sample *xxs, char *buffer)
{
   int i;
   uint8 *buf;
   struct patch_info *patch;

   /* FM patches */

   if (xxs == NULL) {
      patch = Ecalloc(1, sizeof(struct patch_info) + 11);

      memcpy(&patch->data, buffer, 11);
      patch->len = -1;
      patch->instr_no = id;
      xmp_drv_writepatch(patch);
      Efree(patch);                /* Ugh! */
      return;
   }

   if (id == 0xff)                /* Argh! */
      return;

   /* Empty samples */

   if (xxs->len <= 4) {
      char _b[4];

      if (~flags & XMP_SMP_NOLOAD)
         fread(_b, 1, xxs->len, f);
      return;
   }

   /* Patches with samples */

   if ((xmp_ctl->flags & XMP_CTL_8BIT) && (xxs->flg & WAVE_16_BITS)) {
      /* Convert 16 bit samples to 8 bit */
      patch = Ecalloc(1, xxs->len / 2 + sizeof(struct patch_info) + sizeof(int));

      buf = Ecalloc(1, xxs->len);

      if (flags & XMP_SMP_NOLOAD)
         memcpy(buf, buffer, xxs->len);
      else
         fread(buf, 1, xxs->len, f);

#ifdef WORDS_BIGENDIAN
      if (~xmp_ctl->flags & XMP_CTL_BIGEND)
         change_sex(xxs->len, (char *) buf);
#else
      if (xmp_ctl->flags & XMP_CTL_BIGEND)
         change_sex(xxs->len, (char *) buf);
#endif

      if (flags & XMP_SMP_DIFF)
         diff2abs(xxs->len, xxs->flg & WAVE_16_BITS, (char *) buf);
      else if (flags & XMP_SMP_8BDIFF)
         diff2abs(xxs->len, 0, (char *) buf);

      xxs->len /= 2;
      xxs->lpe /= 2;
      xxs->lps /= 2;

      for (i = 0; i < xxs->len; i++)
         buf[i] = ((uint16 *) buf)[i] >> 8;

      memcpy(patch->data, buf, xxs->len);
      Efree(buf);
      xxs->flg &= ~WAVE_16_BITS;
   }
   else {
      patch = Ecalloc(1, sizeof(struct patch_info) + sizeof(int) +
                        (flags & XMP_SMP_8X ? xxs->len * 8 : xxs->len));

      if (flags & XMP_SMP_NOLOAD)
         memcpy(patch->data, buffer, xxs->len);
      else {
         int x;
         char s[5];

         x = ftell(f);
         fread(s, 1, 5, f);
         fseek(f, x, SEEK_SET);

         /* Test for ADPCM encoded samples - these are usually found on
          * "MDZ" files (zipped M.K.)
          */
         if (!strncmp(s, "ADPCM", 5)) {
            int x2 = xxs->len / 2;
            char table[16];
            struct adpcm_state state;

            fseek(f, 5, SEEK_CUR);        /* Skip "ADPCM" */
            fread(table, 1, 16, f);
            fread(patch->data + x2, 1, x2, f);
            adpcm_decoder((unsigned char *) (patch->data + x2),
                          (unsigned char *) patch->data, table, xxs->len, &state);
         }
         else {
            fread(patch->data, 1, xxs->len, f);

#ifdef WORDS_BIGENDIAN
            if (!(xmp_ctl->flags & XMP_CTL_BIGEND) &&
                xxs->flg & WAVE_16_BITS)
               change_sex(patch->len, patch->data);
#else
            if ((xmp_ctl->flags & XMP_CTL_BIGEND) &&
                xxs->flg & WAVE_16_BITS)
               change_sex(patch->len, patch->data);
#endif
         }
      }

      /* This is an attempt to get a better phase adjustment for MED
       * synth sounds and GUS interpolation.
       */
      if (flags & XMP_SMP_8X) {
         int i, j;

         for (i = xxs->len - 1; i >= 0; i--)
            for (j = 7; j >= 0; j--)
               patch->data[i * 8 + j] = patch->data[i];
         xxs->len *= 8;
         xxs->lps *= 8;
         xxs->lpe *= 8;
      }

      if (flags & XMP_SMP_7BIT)
         double_sample(xxs->len, patch->data);
      if (flags & XMP_SMP_DIFF)
         diff2abs(xxs->len, xxs->flg & WAVE_16_BITS, patch->data);
      else if (flags & XMP_SMP_8BDIFF)
         diff2abs(xxs->len, 0, patch->data);
   }

   patch->key = GUS_PATCH;
   patch->instr_no = id;
   patch->mode = xxs->flg;
   patch->mode |= (flags & XMP_SMP_UNS) ? WAVE_UNSIGNED : 0;
   patch->len = xxs->len;
   patch->loop_start = (xxs->lps > xxs->len) ? xxs->len : xxs->lps;
   patch->loop_end = (xxs->lpe > xxs->len) ? xxs->len : xxs->lpe;
   if (patch->loop_end <= patch->loop_start)
      patch->mode &= ~WAVE_LOOPING;

   /* New routine to minimize loop clicking */

   if ((patch->mode & WAVE_LOOPING) && !(patch->mode & WAVE_BIDIR_LOOP)) {
      if (patch->mode & WAVE_16_BITS) {
         patch->data[patch->loop_end] = patch->data[patch->loop_start];
         patch->data[patch->loop_end + 1] =
            patch->data[patch->loop_start + 1];
         patch->len += 2;
      }
      else {
         patch->data[patch->loop_end] = patch->data[patch->loop_start];
         patch->len += 1;
      }
   }
   else {
      if (patch->mode & WAVE_16_BITS) {
         patch->data[patch->len] = patch->data[patch->len - 2];
         patch->data[patch->len + 1] = patch->data[patch->len - 1];
         patch->len += 2;
      }
      else {
         patch->data[patch->len] = patch->data[patch->len - 1];
         patch->len += 1;
      }
   }

   patch->base_note = C4_FREQ;
   patch->base_freq = basefreq;
   patch->high_note = 0x7fffffff;
   patch->low_note = 0;
   patch->volume = 120;
   patch->panning = 0;
   patch->detuning = 0;

   xmp_drv_writepatch(patch);
}


struct xmp_drv_info *xmp_drv_array()
{
   return drv_array;
}
