/* Extended Module Player
 * Copyright (C) 1996-1999 Claudio Matsuoka and Hipolito Carraro Jr
 *
 * This file is part of the Extended Module Player and is distributed
 * under the terms of the GNU General Public License. See docs/COPYING
 * for more information.
 * 
 * Hacked by Isaac Richards to support eMusic.
 */

#include "config.h"

#include <string.h>
#include <stdlib.h>
#include <sys/time.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <unistd.h>
#include <sys/ioctl.h>

#include "driver.h"
#include "mixer.h"

static void bufdump(int);
static void shutdown(void);
static void flush(void);
static int init(struct xmp_control *);
static void setaudio(struct xmp_control *);

static void dummy()
{
}

static char *help[] =
{
   NULL
};

struct xmp_drv_info drv_emusic =
{
   "emusic",                        /* driver ID */
   "eMusic Audio",                /* driver description */
   help,                        /* help */
   init,                        /* init */
   shutdown,                        /* shutdown */
   xmp_smix_numvoices,                /* numvoices */
   dummy,                        /* voicepos */
   xmp_smix_echoback,                /* echoback */
   dummy,                        /* setpatch */
   xmp_smix_setvol,                /* setvol */
   dummy,                        /* setnote */
   xmp_smix_setpan,                /* setpan */
   dummy,                        /* setbend */
   dummy,                        /* seteffect */
   dummy,                        /* starttimer */
   flush,                        /* stctlimer */
   dummy,                        /* reset */
   bufdump,                        /* bufdump */
   dummy,                        /* bufwipe */
   dummy,                        /* clearmem */
   dummy,                        /* sync */
   xmp_smix_writepatch,                /* writepatch */
   xmp_smix_getmsg,                /* getmsg */
   NULL
};

static int do_sync = 1;

static void setaudio(struct xmp_control *ctl)
{
   int sig = 0, big = 0, six = 1, mono = 0;

   if (ctl->outfmt & XFMT_U8 || ctl->outfmt & XFMT_S8)
      six = 0;
   if (ctl->outfmt & (XFMT_S8 | XFMT_S16_LE | XFMT_S16_BE))
      sig = 1;
   if (ctl->outfmt & (XFMT_U16_BE | XFMT_S16_BE))
      big = 1;
   mono = ctl->flags & XMP_CTL_MONO ? 0 : 1;
   setup_audio(ctl->freq, mono, sig, big, six);
}


static int init(struct xmp_control *ctl)
{
   setaudio(ctl);
   return xmp_smix_init(ctl);
}


/* Build and write one tick (one PAL frame or 1/50 s in standard vblank
 * timed mods) of audio data to the output device.
 */
static void bufdump(int i)
{
   for (; i -= audio_play(xmp_smix_buffer(), i););
}


static void shutdown()
{
   xmp_smix_close();
}


static void flush()
{
   if (!do_sync)
      return;
}
