

#include "xmpi.h"

typedef int32 LONG;
typedef uint32 ULONG;
typedef int16 WORD;
typedef uint16 UWORD;
typedef int8 BYTE;
typedef uint8 UBYTE;


/* Structures as defined in the MED/OctaMED MMD0 and MMD1 file formats,
 * revision 1, described by Teijo Kinnunen in Apr 25 1992
 */

struct MMD0sample {
   UWORD rep, replen;                /* offs: 0(s), 2(s) */
   UBYTE midich;                /* offs: 4(s) */
   UBYTE midipreset;                /* offs: 5(s) */
   UBYTE svol;                        /* offs: 6(s) */
   BYTE strans;                        /* offs: 7(s) */
} PACKED;


struct MMD0song {
   struct MMD0sample sample[63];        /* 63 * 8 bytes = 504 bytes */
   UWORD numblocks;                /* offs: 504 */
   UWORD songlen;                /* offs: 506 */
   UBYTE playseq[256];                /* offs: 508 */
   UWORD deftempo;                /* offs: 764 */
   BYTE playtransp;                /* offs: 766 */
#define FLAG_FILTERON   0x1        /* the hardware audio filter is on */
#define FLAG_JUMPINGON  0x2        /* mouse pointer jumping on */
#define FLAG_JUMP8TH    0x4        /* ump every 8th line (not in OctaMED Pro) */
#define FLAG_INSTRSATT  0x8        /* sng+samples indicator (not useful in MMDs) */
#define FLAG_VOLHEX        0x10        /* volumes are HEX */
#define FLAG_STSLIDE    0x20        /* use ST/NT/PT compatible sliding */
#define FLAG_8CHANNEL   0x40        /* this is OctaMED 5-8 channel song */
   UBYTE flags;                        /* offs: 767 */
#define FLAG2_BMASK        0x1F        /* (bits 0-4) BPM beat length (in lines) */
#define FLAG2_BPM       0x20        /* BPM mode on */
   UBYTE flags2;                /* offs: 768 */
   UBYTE tempo2;                /* offs: 769 */
   UBYTE trkvol[16];                /* offs: 770 */
   UBYTE mastervol;                /* offs: 786 */
   UBYTE numsamples;                /* offs: 787 */
} PACKED;                        /* length = 788 bytes */


struct MMD0 {
   ULONG id;
   ULONG modlen;
   struct MMD0song *song;
   UWORD psecnum;                /* defined in MMD2 */
   UWORD pseq;                        /* defined in MMD2 */
   struct MMD0Block **blockarr;
   ULONG reserved1;
   struct InstrHdr **smplarr;
   ULONG reserved2;
   struct MMD0exp *expdata;
   ULONG reserved3;
   UWORD pstate;                /* some data for the player routine */
   UWORD pblock;
   UWORD pline;
   UWORD pseqnum;
   WORD actplayline;
   UBYTE counter;
   UBYTE extra_songs;                /* number of songs - 1 */
} PACKED;                        /* length = 52 bytes */


struct MMD0Block {
   UBYTE numtracks, lines;
} PACKED;


struct BlockInfo {
   ULONG *hlmask;
   UBYTE *blockname;
   ULONG blocknamelen;
   ULONG reserved[6];
} PACKED;


struct MMD1Block {
   UWORD numtracks;
   UWORD lines;
   struct BlockInfo *info;
} PACKED;


struct InstrHdr {
   ULONG length;
   WORD type;
   /* Followed by actual data */
} PACKED;


struct SynthWF {
   UWORD length;                /* length in words */
   BYTE wfdata[1];                /* the waveform */
} PACKED;


struct SynthInstr {
   ULONG length;                /* length of this struct */
   WORD type;                        /* -1 or -2 (offs: 4) */
   UBYTE defaultdecay;
   UBYTE reserved[3];
   UWORD rep;
   UWORD replen;
   UWORD voltbllen;                /* offs: 14 */
   UWORD wftbllen;                /* offs: 16 */
   UBYTE volspeed;                /* offs: 18 */
   UBYTE wfspeed;                /* offs: 19 */
   UWORD wforms;                /* offs: 20 */
   UBYTE voltbl[128];                /* offs: 22 */
   UBYTE wftbl[128];                /* offs: 150 */
   struct SynthWF *wf[64];        /* offs: 278 */
} PACKED;


struct InstrExt {
   UBYTE hold;
   UBYTE decay;
   UBYTE suppress_midi_off;
   BYTE finetune;
} PACKED;


struct MMD0exp {
   struct MMD0 *nextmod;
   struct InstrExt *exp_smp;
   UWORD s_ext_entries;
   UWORD s_ext_entrsz;
   UBYTE *annotxt;
   ULONG annolen;
   struct MMDInstrInfo *iinfo;
   UWORD i_ext_entries;
   UWORD i_ext_entrsz;
   ULONG jumpmask;
   UWORD *rgbtable;
   UBYTE channelsplit[4];
   struct NotationInfo *n_info;
   UBYTE *songname;
   ULONG songnamelen;
   struct MMDDumpData *dumps;
   ULONG reserved2[7];
} PACKED;


struct NotationInfo {
   UBYTE n_of_sharps;
   UBYTE flags;
   WORD trksel[5];
   UBYTE trkshow[16];
   UBYTE trkghost[16];
   BYTE notetr[63];
   UBYTE pad;
} PACKED;


struct MMDDumpData {
   UWORD numdumps;
   UWORD reserved[3];
} PACKED;


struct MMDDump {
   ULONG length;
   UBYTE *data;
   UWORD ext_len;
   /* if ext_len >= 20: */
   UBYTE name[20];
} PACKED;
