
/* Extended Module Player 
 * Copyright (C) 1996-1999 Claudio Matsuoka and Hipolito Carraro Jr
 * $Id: load.c,v 1.3 1998/11/04 15:24:31 claudio Exp $.
 *
 * This file is part of the Extended Module Player and is distributed
 * under the terms of the GNU General Public License. See doc/COPYING
 * for more information.
 */

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/stat.h>
#include <unistd.h>
#include "driver.h"

extern struct xmp_fmt_info *__fmt_head;

int xmpi_decrunch_sqsh(FILE *, char *);
int xmpi_decrunch_pp(FILE *, char *);

#define BUILTIN_PP    0x01
#define BUILTIN_SQSH  0x02

static int decrunch(FILE ** f, char *s, char **temp)
{
   unsigned char b[64];
   char *cmd, *line;
   FILE *t;
   int builtin, res;
   char *temp2, *packer;

   fread(b, 1, 64, *f);

   packer = cmd = NULL;
   builtin = res = 0;

   if (b[0] == 'P' && b[1] == 'K') {
      packer = "Zip";
      cmd = "unzip -pqqC \"%s\" -x readme '*.diz' '*.nfo' '*.txt' "
         "'*.exe' '*.com' 2>/dev/null>%s";
   }
   else if (b[2] == '-' && b[3] == 'l' && b[4] == 'h') {
      packer = "LHa";
      cmd = "lha -pq \"%s\" > %s";
   }
   else if (b[0] == 31 && b[1] == 139) {
      packer = "gzip";
      cmd = "gzip -dc \"%s\" > %s";
   }
   else if (b[0] == 'B' && b[1] == 'Z' && b[2] == 'h') {
      packer = "bzip2";
      cmd = "bzip2 -dc \"%s\" > %s";
   }
   else if (b[0] == 31 && b[1] == 157) {
      packer = "compress";
      cmd = "uncompress -c \"%s\" > %s";
   }
   else if (b[0] == 'P' && b[1] == 'P' && b[2] == '2' && b[3] == '0') {
      packer = "PowerPack";
      builtin = BUILTIN_PP;
   }
   else if (b[0] == 'X' && b[1] == 'P' && b[2] == 'K' && b[3] == 'F' &&
            b[8] == 'S' && b[9] == 'Q' && b[10] == 'S' && b[11] == 'H') {
      packer = "SQSH";
      builtin = BUILTIN_SQSH;
   }
   else if (b[0] == 'z' && b[1] == 'i' && b[2] == 'R' && b[3] == 'C' &&
            b[4] == 'O' && b[5] == 'N' && b[6] == 'i' && b[7] == 'a') {
      if (xmp_ctl->verbose)
         report("Can't decompress MMCMP modules (Zirconia help me!)\n");
   }

   fseek(*f, 0, SEEK_SET);

   if (!packer)
      return 0;

   if (xmp_ctl->verbose)
      report("Decrunching %s file... ", packer);

   if (cmd) {
      line = Emalloc(strlen(cmd) + strlen(s) + strlen(*temp) + 16);
      sprintf(line, cmd, s, *temp);

      if (system(line)) {
         if (xmp_ctl->verbose)
            report("failed\n");
         Efree(line);
         return -1;
      }
      Efree(line);
   }
   else
      switch (builtin) {
      case BUILTIN_PP:
         res = xmpi_decrunch_pp(*f, *temp);
         break;
      case BUILTIN_SQSH:
         res = xmpi_decrunch_sqsh(*f, *temp);
         break;
      }

   if (res < 0 || (t = fopen(*temp, "r")) == NULL) {
      if (xmp_ctl->verbose)
         report("failed\n");
      return -1;
   }

   if (xmp_ctl->verbose)
      report("done\n");

   fclose(*f);
   *f = t;

   temp2 = tempnam(NULL, "xmp_");
   res = decrunch(f, *temp, &temp2);
   if (res) {
      unlink(*temp);
      free(*temp);
      *temp = temp2;
   }

   return 1;
}


static void get_smp_size(int awe, int *a, int *b)
{
   int i, len, smp_size, smp_4kb;

   for (smp_4kb = smp_size = i = 0; i < xxh->smp; i++) {
      len = xxs[i].len;

      /* AWE cards work with 16 bit samples only and expand bidirectional
       * loops.
       */
      if (awe) {
         if (xxs[i].flg & WAVE_BIDIR_LOOP)
            len += xxs[i].lpe - xxs[i].lps;
         if (~xxs[i].flg & WAVE_16_BITS)
            len <<= 1;
      }

      smp_size += len;
      if (len < 0x1000)
         smp_4kb += len;
   }

   *a = smp_size;
   *b = smp_4kb;
}

static int crunch_ratio(void)
{
   int memavl, smp_size, ratio, smp_4kb, awe;

   ratio = 0x10000;
   if (!(memavl = xmp_ctl->memavl))
      return ratio;

   awe = (strstr(xmp_ctl->description, " [AWE") != NULL);

   get_smp_size(awe, &smp_size, &smp_4kb);

   if (smp_size > memavl) {
      if (!awe)
         xmp_drv_cvt8bit();
      get_smp_size(awe, &smp_size, &smp_4kb);
   }

   if (xmp_ctl->crunch || smp_size > memavl) {
      ratio = xmp_ctl->crunch ? xmp_ctl->crunch :
         (0x100 * (memavl - smp_4kb) / (smp_size - smp_4kb)) * 0x100;
      if (xmp_ctl->verbose)
         report("Crunch ratio   : %d%% [Mem:%.3fMb Smp:%.3fMb]\n",
                100 * ratio / 0x10000, .000001 * memavl, .000001 * smp_size);
   }

   return ratio;
}


int xmp_load_module(char *s)
{
   FILE *f;
   int i, t;
   char *temp;
   struct xmp_fmt_info *fmt;
   struct stat st;
   unsigned int crc;

   if ((f = fopen(s, "r")) == NULL)
      return -3;

   if (fstat(fileno(f), &st) < 0)
      return -3;

   if (S_ISDIR(st.st_mode))
      return -1;

   temp = tempnam(NULL, "xmp_");
   if ((t = decrunch(&f, s, &temp)) < 0) {
      unlink(temp);
      Efree(temp);
      return -1;
   }

   if (fstat(fileno(f), &st) < 0)        /* Size after decrunch */
      return -3;

   crc = cksum(f);

   xmp_drv_clearmem();

   /* Reset variables */
   memset(xmp_ctl->name, 0, XMP_DEF_NAMESIZE);
   memset(xmp_ctl->type, 0, XMP_DEF_NAMESIZE);
   xmp_ctl->filename = s;        /* For ALM */
   xmp_ctl->size = st.st_size;
   xmp_ctl->rrate = PAL_RATE;
   xmp_ctl->c4rate = C4_PAL_RATE;
   xmp_ctl->volbase = 0x40;
   xmp_ctl->volume = 0x40;
   xmp_ctl->vol_xlat = NULL;
   xmp_ctl->flags &= ~(XMP_CTL_VSALL | XMP_CTL_FINEFX | XMP_CTL_MEDBPM |
                       XMP_CTL_VIRTUAL | XMP_CTL_FMASK);

   xmpi_read_modconf(xmp_ctl, crc, st.st_size);

   xxh = Ecalloc(sizeof(struct xxm_header), 1);

   /* Set defaults */
   xxh->tpo = 6;
   xxh->bpm = 125;
   xxh->chn = 4;

   for (i = 0; i < 64; i++) {
      xxc[i].pan = (((i + 1) / 2) % 2) * 0xff;
      xxc[i].cho = xmp_ctl->chorus;
      xxc[i].rvb = xmp_ctl->reverb;
   }

   for (i = 0, fmt = __fmt_head; fmt; fmt = fmt->next) {
      if (fmt->loader && ((i = fmt->loader(f)) != -1))
         break;
   }

   fclose(f);

   if (t)
      unlink(temp);

   free(temp);

   if (i < 0)
      return i;

   xmp_drv_flushpatch(crunch_ratio());

   str_adj(xmp_ctl->name);
   if (!*xmp_ctl->name)
      strcpy(xmp_ctl->name, "(untitled)");

   if (xmp_ctl->verbose > 1) {
      report("Module looping : %s\n",
             xmp_ctl->flags & XMP_CTL_LOOP ? "yes" : "no");
      report("Period mode    : %s\n",
             xxh->flg & XXM_FLG_LINEAR ? "linear" : "Amiga");
   }

   if (xmp_ctl->verbose > 2) {
      report("Restart pos    : %d\n", xxh->rst);
      report("Base volume    : %d\n", xmp_ctl->volbase);
      report("C4 replay rate : %d\n", xmp_ctl->c4rate);
      report("Channel mixing : %d%% (dynamic pan %s)\n",
             xmp_ctl->flags & XMP_CTL_REVERSE ? -xmp_ctl->mix : xmp_ctl->mix,
             xmp_ctl->flags & XMP_CTL_DYNPAN ? "enabled" : "disabled");
   }

   if (xmp_ctl->verbose) {
      report("Channels       : %d [ ", xxh->chn);
      for (i = 0; i < xxh->chn; i++) {
         if (xxc[i].flg & XXM_CHANNEL_FM)
            report("F ");
         else
            report("%x ", xxc[i].pan >> 4);
      }
      report("]\n");
   }

   t = xmpi_scan_module();

   if (xmp_ctl->verbose) {
      if (xmp_ctl->flags & XMP_CTL_LOOP)
         report("One loop time  : %dmin%02ds\n",
                (t + 500) / 60000, ((t + 500) / 1000) % 60);
      else
         report("Estimated time : %dmin%02ds\n",
                (t + 500) / 60000, ((t + 500) / 1000) % 60);
   }

   return t;
}
