
#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include "xmpi.h"
#include "driver.h"
#include "mixer.h"

extern int *tbuffer;
extern int Itpt_inc;
extern int vol_right;
extern int vol_left;

/* Mixers

 * To increase performance eight mixers are defined, one for each
 * combination of the following parameters: interpolation, resolution
 * and number of channels.
 */
#define INTERPOLATE() \
    if (itpt >> SHIFT_FT) { \
        cur_bk += itpt >> SHIFT_FT; \
        smp_x1 = in_bk[cur_bk]; \
        smp_dt = in_bk[cur_bk + 1] - smp_x1; \
        itpt &= AND_FT; \
    } \
    smp_in = smp_x1 + ((itpt * smp_dt) >> SHIFT_FT);

#define DONT_INTERPOLATE() \
    smp_in = in_bk[itpt >> SHIFT_FT];


/* Handler for 8 bit samples, interpolated stereo output */
void mix_st8itpt(int count, char *in_bk, int cur_bk, register int itpt)
{
   int vl, vr;
   int smp_x1 = 0, smp_dt = 0;
   register int smp_in;
   register int *tmp_bk = tbuffer;
   int itpt_inc = Itpt_inc;

   vl = vol_left << 8;
   vr = vol_right << 8;
   while (count--) {
      INTERPOLATE();
      *(tmp_bk++) += smp_in * vr;
      *(tmp_bk++) += smp_in * vl;
      itpt += itpt_inc;
   }
}


/* Handler for 16 bit samples, interpolated stereo output */
void mix_st16itpt(int count, int16 * in_bk, int cur_bk, register int itpt)
{
   int vl, vr;
   int smp_x1 = 0, smp_dt = 0;
   register int smp_in;
   register int *tmp_bk = tbuffer;
   int itpt_inc = Itpt_inc;

   vl = vol_left;
   vr = vol_right;
   while (count--) {
      INTERPOLATE();
      *(tmp_bk++) += smp_in * vr;
      *(tmp_bk++) += smp_in * vl;
      itpt += itpt_inc;
   }
}


/* Handler for 8 bit samples, non-interpolated stereo output */
void mix_st8norm(int count, char *in_bk, int cur_bk, register int itpt)
{
   int vl, vr;
   register int smp_in;
   register int *tmp_bk = tbuffer;
   int itpt_inc = Itpt_inc;

   in_bk += cur_bk;
   vl = vol_left << 8;
   vr = vol_right << 8;
   while (count--) {
      DONT_INTERPOLATE();
      *(tmp_bk++) += smp_in * vr;
      *(tmp_bk++) += smp_in * vl;
      itpt += itpt_inc;
   }
}


/* Handler for 16 bit samples, non-interpolated stereo output */
void mix_st16norm(int count, int16 * in_bk, int cur_bk, register int itpt)
{
   int vl, vr;
   register int smp_in;
   register int *tmp_bk = tbuffer;
   int itpt_inc = Itpt_inc;

   in_bk += cur_bk;
   vl = vol_left;
   vr = vol_right;
   while (count--) {
      DONT_INTERPOLATE();
      *(tmp_bk++) += smp_in * vr;
      *(tmp_bk++) += smp_in * vl;
      itpt += itpt_inc;
   }
}


/* Handler for 8 bit samples, interpolated mono output */
void mix_mn8itpt(int count, char *in_bk, int cur_bk, register int itpt)
{
   int smp_x1 = 0, smp_dt = 0, vl;
   register int smp_in;
   register int *tmp_bk = tbuffer;
   int itpt_inc = Itpt_inc;

   vl = vol_left << 9;
   while (count--) {
      INTERPOLATE();
      *(tmp_bk++) += smp_in * vl;
      itpt += itpt_inc;
   }
}


/* Handler for 16 bit samples, interpolated mono output */
void mix_mn16itpt(int count, int16 * in_bk, int cur_bk, register int itpt)
{
   int smp_x1 = 0, smp_dt = 0, vl;
   register int smp_in;
   register int *tmp_bk = tbuffer;
   int itpt_inc = Itpt_inc;

   vl = vol_left << 1;
   while (count--) {
      INTERPOLATE();
      *(tmp_bk++) += smp_in * vl;
      itpt += itpt_inc;
   }
}


/* Handler for 8 bit samples, non-interpolated mono output */
void mix_mn8norm(int count, char *in_bk, int cur_bk, register int itpt)
{
   int vl;
   register int smp_in;
   register int *tmp_bk = tbuffer;
   int itpt_inc = Itpt_inc;

   in_bk += cur_bk;
   vl = vol_left << 9;
   while (count--) {
      DONT_INTERPOLATE();
      *(tmp_bk++) += smp_in * vl;
      itpt += itpt_inc;
   }
}


/* Handler for 16 bit samples, non-interpolated mono output */
void mix_mn16norm(int count, int16 * in_bk, int cur_bk, register int itpt)
{
   int vl;
   register int smp_in;
   register int *tmp_bk = tbuffer;
   int itpt_inc = Itpt_inc;

   in_bk += cur_bk;
   vl = vol_left << 1;
   while (count--) {
      DONT_INTERPOLATE();
      *(tmp_bk++) += smp_in * vl;
      itpt += itpt_inc;
   }
}
