
#include "mod.h"
#include <string.h>
#include <stdlib.h>

#include "xmp.h"
static struct xmp_module_info mi;
static int mod_length = 0, tpo, bpm;
static double seconds;

static int mod_playing = FALSE;
static int mod_init = FALSE;
static struct xmp_control opt;
extern struct xmp_control ctlrc;

void *mod_thread(void *foo);

static void player_callback(unsigned long i)
{
   unsigned long msg = i >> 4;
   static int row, pos, pat;

   switch (i & 0xf) {
   case XMP_ECHO_BPM:
      bpm = msg & 0xff;
      tpo = msg >> 8;
      break;
   case XMP_ECHO_ORD:
      pos = msg & 0xff;
      pat = msg >> 8;
      break;
   case XMP_ECHO_ROW:
      row = msg & 0xff;
      seconds += 60.0 * tpo / (24 * bpm);
      break;
   }
}

void xmp_seek_to_frame(int second)
{
   if (second > seconds)
      xmp_ord_next();
   if (second < seconds)
      xmp_ord_prev();
}


int xmp_play(char *song)
{
   if (!song) {
      fprintf(stderr, "No playlist loaded\n");
      return 0;
   }

   xmp_init(0, NULL, &opt);
   opt.verbose = 0;
   xmp_register_event_callback(player_callback);

   if (xmp_open_audio(&opt) < 0) {
      fprintf(stderr, "Can't open audio device");
      return 0;
   }

   if ((mod_length = xmp_load_module(song)) < 0) {
      fprintf(stderr, "Error loading file: %s\n", song);
      xmp_close_audio();
      return 0;
   }

   xmp_get_module_info(&mi);
   seconds = 0;
   if (strlen(mi.name)) {
      cur_item->name = Erealloc(cur_item->name, strlen(mi.name) + strlen(cur_item->name));
      strcpy((char *) (strchr(cur_item->name, '-') + 2), mi.name);
   }
   cur_item->length = mod_length / 1000;

   pthread_create(&player_thread, NULL, mod_thread, NULL);
   return 1;
}


void xmp_pause_it(void)
{
   xmp_mod_pause();
}


void *mod_thread(void *foo)
{
   mod_playing = TRUE;
   mod_init = TRUE;
   mod_length = xmp_play_module();
   end_song = TRUE;
   mod_playing = FALSE;
   return NULL;
}

void xmp_stop(int playing)
{
   if (!mod_init)
      return;
   xmp_stop_module();
   while (mod_playing)
      usleep(500);
   pthread_join(player_thread, NULL);
   xmp_close_audio();
   end_song = FALSE;
   mod_init = FALSE;
}

void xmp_ini(struct player_plugin *plug)
{
   static int registered = FALSE;

   if (!registered) {
      registered = TRUE;
      emusic_register_ext("xm", plug);
      emusic_register_ext("mod", plug);
      emusic_register_ext("it", plug);
      emusic_register_ext("s3m", plug);
      emusic_register_ext("stm", plug);
      emusic_register_ext("stx", plug);
      emusic_register_ext("mtm", plug);
      emusic_register_ext("imf", plug);
      emusic_register_ext("ptm", plug);
      emusic_register_ext("mdl", plug);
      emusic_register_ext("ult", plug);
      emusic_register_ext("liq", plug);
      emusic_register_ext("ptm", plug);
      emusic_register_ext("med", plug);
      emusic_register_ext("dbm", plug);
      emusic_register_ext("emod", plug);
      emusic_register_ext("okt", plug);
      emusic_register_ext("sfx", plug);
      emusic_register_ext("far", plug);
      emusic_register_ext("stim", plug);
      emusic_register_ext("fc-m", plug);
      emusic_register_ext("wn", plug);
      emusic_register_ext("pm", plug);
      emusic_register_ext("kris", plug);
      emusic_register_ext("unic", plug);
      emusic_register_ext("lax", plug);
      emusic_register_ext("tp60", plug);
      emusic_register_ext("pru1", plug);
      emusic_register_ext("pru2", plug);
      emusic_register_ext("pm01", plug);
      emusic_register_ext("pm10", plug);
      emusic_register_ext("pm18", plug);
      emusic_register_ext("pm20", plug);
      emusic_register_ext("pm40", plug);
      emusic_register_ext("ac1d", plug);
      emusic_register_ext("pp10", plug);
      emusic_register_ext("xann", plug);
      emusic_register_ext("np1", plug);
      emusic_register_ext("np2", plug);
      emusic_register_ext("np3", plug);
      emusic_register_ext("di", plug);
      emusic_register_ext("mp", plug);
      emusic_register_ext("669", plug);
#ifdef DRIVER_OSS_SEQ
      emusic_register_ext("amd", plug);
      emusic_register_ext("rad", plug);
#endif
      emusic_register_ext("alm", plug);
      emusic_register_ext("Z", plug);
      emusic_register_ext("zip", plug);
      emusic_register_ext("lha", plug);
      emusic_register_ext("lzh", plug);
      emusic_register_ext("bz2", plug);
      emusic_register_ext("mdz", plug);
      emusic_register_ext("gz", plug);
   }
}

void xmp_get_info(struct playlist_item *song)
{
   song->length = mod_length / 1000;
}

struct player_plugin *setup_plugin(void)
{
   struct player_plugin *temp = Emalloc(sizeof(struct player_plugin));

   memset(&ctlrc, 0, sizeof(struct xmp_control));

   temp->init = xmp_ini;
   temp->config = xmp_config;
   temp->seek_to_frame = xmp_seek_to_frame;
   temp->play = xmp_play;
   temp->stop = xmp_stop;
   temp->pause_it = xmp_pause_it;
   temp->get_info = xmp_get_info;
   temp->id = duplicate("xmp");
   temp->streamable = FALSE;
   temp->version = duplicate("2.0.0dev36");
   temp->copyright =
      duplicate("1996-1999 Claudio Matsuoka and Hipolito Carraro Jr");
   return temp;
}
