
/* Extended Module Player - readrc.c
 * Copyright (C) 1996-1998 Claudio Matsuoka and Hipolito Carraro Jr
 *
 * This file is part of the Extended Module Player and is distributed
 * under the terms of the GNU General Public License. See doc/COPYING
 * for more information.
 */

#ifdef HVAE_CONFIG_H
#include "config.h"
#endif

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include "xmpi.h"
#include "../common.h"

extern int __mode_fm;
static int smp8bit = 0;
int _chorusmode;
int _reverbmode;
static char *drv_id;

struct xmp_control ctlrc;

static int get_yesno(char *s)
{
   return !(strncmp(s, "y", 1) && strncmp(s, "o", 1));
}


int xmpi_read_rc(struct xmp_control *opt)
{
   opt->flags |= ctlrc.flags;
   if (ctlrc.verbose)
      opt->verbose = ctlrc.verbose;
   if (ctlrc.mix)
      opt->mix = ctlrc.mix;
   if (ctlrc.freq)
      opt->freq = ctlrc.freq;
   if (ctlrc.time)
      opt->time = ctlrc.time;
   if (ctlrc.crunch)
      opt->crunch = ctlrc.crunch;
   if (ctlrc.chorus)
      opt->chorus = ctlrc.chorus;
   if (ctlrc.reverb)
      opt->reverb = ctlrc.reverb;
   if (smp8bit)
      opt->outfmt = XFMT_S8;
   else
#ifdef WORDS_BIGENDIAN
      opt->outfmt = XFMT_S16_BE;
#else
      opt->outfmt = XFMT_S16_LE;
#endif
   return 0;
}

void xmp_config(char *var, char *val)
{
#define getval_yn(x,y) { \
        if (!strcasecmp(var,x)) { if (get_yesno (val)) ctlrc.flags |= (y); \
            else ctlrc.flags &= ~(y); return; } }

#define getval_no(x,y) { \
        if (!strcasecmp(var,x)) { y = atoi (val); return; } }

   getval_yn("8BIT", XMP_CTL_8BIT);
   getval_yn("INTERPOLATE", XMP_CTL_ITPT);
   getval_yn("LOOP", XMP_CTL_LOOP);
   getval_yn("MONO", XMP_CTL_MONO);
   getval_yn("REVERSE", XMP_CTL_REVERSE);
   getval_yn("PAN", XMP_CTL_DYNPAN);
   getval_yn("FIXLOOP", XMP_CTL_FIXLOOP);
   getval_yn("FX9BUG", XMP_CTL_FX9BUG);
   getval_no("MIX", ctlrc.mix);
   getval_no("CRUNCH", ctlrc.crunch);
   getval_no("CHORUS", ctlrc.chorus);
   getval_no("REVERB", ctlrc.reverb);
   getval_no("SRATE", ctlrc.freq);
   getval_no("TIME", ctlrc.time);
   getval_no("VERBOSITY", ctlrc.verbose);

#if 0
/****** FIXME ******/
   if (!strcasecmp(var, "BITS")) {
      smp8bit = (atoi(val) == 8);
      return;
   }
#endif
   if (!strcasecmp(var, "awechorus")) {
      sscanf(val, "%d", &_chorusmode);
      return;
   }
   if (!strcasecmp(var, "awereverb")) {
      sscanf(val, "%d", &_reverbmode);
      return;
   }
   if (!strcasecmp(var, "opl2")) {
      __mode_fm = 1;
      return;
   }
   if (!strcasecmp(var, "driver")) {
      drv_id = duplicate(val);
      ctlrc.drv_id = drv_id;
      return;
   }
}


static void parse_modconf(struct xmp_control *ctl, char *fn, unsigned int crc, unsigned int size)
{
   FILE *rc;
   char *hash, *var, *val, line[256];
   int active = 0;

   if ((rc = fopen(fn, "r")) == NULL)
      return;

   while (!feof(rc)) {
      memset(line, 0, 256);
      fscanf(rc, "%255[^\n]", line);
      fgetc(rc);

      /* Delete comments */
      if ((hash = strchr(line, '#')))
         *hash = 0;

      if (line[0] == ':') {
         strtok(&line[1], " ");
         val = strtok(NULL, " \t\n");
         if (strtoul(&line[1], NULL, 0) && strtoul(val, NULL, 0)) {
            active = (strtoul(&line[1], NULL, 0) == crc &&
                      strtoul(val, NULL, 0) == size);
            if (active && xmp_ctl->verbose > 2)
               report("Matching CRC in %s (%u)\n", fn, crc);
         }
         continue;
      }

      if (!active)
         continue;

      delete_spaces(line);

      if (!(var = strtok(line, "=\n")))
         continue;

      val = strtok(NULL, " \t\n");

      getval_yn("8BIT", XMP_CTL_8BIT);
      getval_yn("INTERPOLATE", XMP_CTL_ITPT);
      getval_yn("LOOP", XMP_CTL_LOOP);
      getval_yn("MONO", XMP_CTL_MONO);
      getval_yn("REVERSE", XMP_CTL_REVERSE);
      getval_yn("FIXLOOP", XMP_CTL_FIXLOOP);
      getval_yn("FX9BUG", XMP_CTL_FX9BUG);
      getval_yn("PAN", XMP_CTL_DYNPAN);
      getval_no("MIX", ctlrc.mix);
      getval_no("CRUNCH", ctlrc.crunch);
      getval_no("CHORUS", ctlrc.chorus);
      getval_no("REVERB", ctlrc.reverb);
   }

   fclose(rc);
}

void xmpi_read_modconf(struct xmp_control *ctl, unsigned int crc, unsigned int size)
{
   char *myrc, *home = getenv("HOME");

   myrc = Emalloc(strlen(THEMEDIR) + 16);
   sprintf(myrc, "%s/xmp.modconf", THEMEDIR);
   parse_modconf(ctl, myrc, crc, size);
   myrc = Erealloc(myrc, (home ? strlen(home) : 0) + 20);
   sprintf(myrc, "%s/.emusic/xmp.modconf", home);
   parse_modconf(ctl, myrc, crc, size);
   Efree(myrc);
}
